from __future__ import unicode_literals

from ..operators import IOperator, Operator
from decimal import Decimal
from gocept.reference.collection import InstrumentedSet
from icemac.addressbook.utils import create_and_add
from zope.container.folder import Folder
import pytest
import zope.component
import zope.component.hooks


# Fixtures to set-up infrastructure which are usable in tests:

@pytest.fixture(scope='function')
def folders(addressBookConnectionF):
    """Fixture returning a callable to create a number of folders."""
    def create_folders(quantity):
        """Create `quantity` folders needed for keyword tests."""
        root = addressBookConnectionF.rootFolder
        names = []
        for _ in range(quantity):
            names.append(create_and_add(root, Folder))
        return tuple(root[name] for name in names)
    return create_folders


# Fixtures to help asserting

@pytest.fixture(scope='session')
def evaluate(zcmlS):
    """Fixture returning a callable to do an operation."""
    return _evaluate


@pytest.fixture(scope='function')
def set_evaluate(addressBookConnectionF):
    """Fixture returning a callable to do a set operation."""
    def _set_evaluate(value1, operator_name, value2):
        """Compute a set operation."""
        with zope.component.hooks.site(addressBookConnectionF.rootFolder):
            return _evaluate(
                InstrumentedSet(value1), operator_name, set(value2))
    return _set_evaluate


# Helper functions

def _evaluate(value1, operator_name, value2):
    """Compute an operation."""
    operator = zope.component.getAdapter(value1, IOperator, name=operator_name)
    return operator(value2)


# Tests

def test_operator__Operator____call____1():
    """It raises an NotImplementedError."""
    op = Operator(None)
    with pytest.raises(NotImplementedError):
        op(None)


def test_operator__append__1(evaluate):
    """append() returns value2 if value1 is `None`."""
    assert 'bar' == evaluate(None, 'append', 'bar')


def test_operator__append__2(evaluate):
    """append() returns concatenation for unicode."""
    assert 'foobar' == evaluate('foo', 'append', 'bar')


def test_operator__replace__1(evaluate):
    """replace() returns value2 for unicodes."""
    assert 'bar' == evaluate('foo', 'replace', 'bar')


def test_operator__replace__2(evaluate):
    """replace() returns value2 for ints."""
    assert 2 == evaluate(1, 'replace', 2)


def test_operator__prepend__1(evaluate):
    """prepend() with `None` returns value2."""
    assert 'bar' == evaluate(None, 'prepend', 'bar')


def test_operator__prepend__2(evaluate):
    """prepend() returns the inverse concatenation of `append`."""
    assert 'barfoo' == evaluate('foo', 'prepend', 'bar')


def test_operator__remove_all__1(evaluate):
    """remove-all() removes all occurrences of value2 in value1."""
    assert 'bac' == evaluate('abbaabc', 'remove-all', 'ab')


def test_operator__remove_all__2(evaluate):
    """remove-all() does nothing on `None`."""
    assert None is evaluate(None, 'remove-all', 'a')


def test_operator__remove_first__1(evaluate):
    """remove-first() removes only the first occurrence of value2."""
    assert 'ddab' == evaluate('abddab', 'remove-first', 'ab')


def test_operator__remove_first__2(evaluate):
    """remove-first() does nothing on `None`."""
    assert None is evaluate(None, 'remove-first', 'a')


def test_operator__remove_last__1(evaluate):
    """remove-last() removes only the last occurrence of value2."""
    assert 'abdd' == evaluate('abddab', 'remove-last', 'ab')


def test_operator__remove_last__2(evaluate):
    """remove-last() does nothing on `None`."""
    assert None is evaluate(None, 'remove-last', 'a')


def test_operator__union__1(folders, set_evaluate):
    """union() returns the union of the values."""
    f1, f2, f3 = folders(3)
    assert set([f1, f2, f3]) == set_evaluate([f1, f2], 'union', [f2, f3])


def test_operator__difference__1(folders, set_evaluate):
    """difference() returns the difference of the values."""
    f1, f2, f3, f4 = folders(4)
    assert set([f1, f3]) == set_evaluate(
        [f1, f2, f3, f4], 'difference', [f2, f4])


def test_operator__intersection__1(folders, set_evaluate):
    """intersection() returns the intersection of the values."""
    f1, f2, f3, f4 = folders(4)
    assert set([f2]) == set_evaluate([f1, f2, f3], 'intersection', [f2, f4])


def test_operator__symmetric_difference__1(folders, set_evaluate):
    """symmetric-difference() returns symmetric difference of the values."""
    f1, f2, f3 = folders(3)
    assert set([f1, f3]) == set_evaluate(
        [f1, f2], 'symmetric_difference', [f2, f3])


def test_operator__add__1(evaluate):
    """add() returns value2 if value1 is `None`."""
    assert 2 == evaluate(None, 'add', 2)


@pytest.mark.parametrize("datatype", (int, Decimal))
def test_operator__add__2(evaluate, datatype):
    """add() returns the sum of value1 and value2."""
    assert datatype(3) == evaluate(datatype(1), 'add', datatype(2))


def test_operator__sub__1(evaluate):
    """sub() returns the negative value of value2 if value1 is `None`."""
    assert -2 == evaluate(None, 'sub', 2)


@pytest.mark.parametrize("datatype", (int, Decimal))
def test_operator__sub__2(evaluate, datatype):
    """sub() returns the difference of value1 and value2."""
    assert datatype(-1) == evaluate(datatype(1), 'sub', datatype(2))


def test_operator__mul__1(evaluate):
    """mul() returns 0 on if value1 is `None`."""
    assert 0 == evaluate(None, 'mul', 2)


@pytest.mark.parametrize("datatype", (int, Decimal))
def test_operator__mul__2(evaluate, datatype):
    """mul() returns the product of value1 and value2."""
    assert datatype(6) == evaluate(datatype(3), 'mul', datatype(2))


def test_operator__div__1(evaluate):
    """div() returns 0 if value1 is `None`."""
    assert 0 == evaluate(None, 'div', 2)


@pytest.mark.parametrize("datatype", (int, Decimal))
def test_operator__div__2(evaluate, datatype):
    """div() returns the quotient depending on the data type of the values."""
    assert datatype(10) / datatype(3) == evaluate(
        datatype(10), 'div', datatype(3))



# -*- coding: utf-8 -*-
from icemac.addressbook.testing import pyTestAddressBookFixture, site
from icemac.addressbook.testing import pyTestEmptyZodbFixture, DateTimeClass
from icemac.addressbook.testing import pyTestStackDemoStorage
import gocept.httpserverlayer.wsgi
import gocept.selenium
import gocept.selenium.wd_selenese
import icemac.addressbook
import icemac.addressbook.file.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.principals.principals
import icemac.addressbook.startup
import icemac.addressbook.testing
import icemac.addressbook.utils
import os
import os.path
import pytest
import tempfile
import transaction
import zope.app.wsgi.testlayer
import zope.browserpage.metaconfigure
import zope.component.hooks
import zope.event
import zope.principalregistry.principalregistry
import zope.processlifetime
import zope.testbrowser.wsgi

pytest_plugins = 'icemac.addressbook.fixtures'


# Fixtures to set-up infrastructure which are usable in tests:

@pytest.yield_fixture(scope='function')
def empty_zodb(zodbS):
    """Get the address book from the right demo storage."""
    for connection in pyTestStackDemoStorage(zodbS, 'EmptyZODB'):
        yield connection


@pytest.yield_fixture(scope='function')
def address_book(addressBookConnectionF):
    """Get the address book as site."""
    for address_book in site(addressBookConnectionF):
        yield address_book


@pytest.yield_fixture(scope='function')
def translated_address_book(translationZcmlF):
    """Get the address book as site for translation tests."""
    for zodb in pyTestEmptyZodbFixture():
        for zodb in pyTestAddressBookFixture(zodb, 'TranslatedAddressBook'):
            for conn in pyTestStackDemoStorage(zodb, 'TranslatedAddressBook2'):
                for address_book in site(conn):
                    yield address_book


@pytest.yield_fixture(scope='function')
def person_data(personDataS):
    """Provide predefined person data, see `personDataS`."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            personDataS.zodb, 'PersonFunction'):
        for addressbook in site(connection):
            yield addressbook


@pytest.fixture(scope='function')
def browser(browserWsgiAppS):
    """Fixture for testing with zope.testbrowser."""
    assert icemac.addressbook.testing.CURRENT_CONNECTION is not None, \
        "The `browser` fixture needs a database fixture like `address_book`."
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.fixture(scope='function')
def browser2(browserWsgiAppS):
    """Fixture for testing with a second zope.testbrowser."""
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.fixture(scope='function')
def browser3(browserWsgiAppS):
    """Fixture for testing with a third zope.testbrowser."""
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.yield_fixture(scope='function')
def tmpfile():
    """Fixture to create a temporary file with a defined content and suffix.

    Returns a callable to create the file which returns a tuple of file handle
    and file name.

    """
    data = {'filename': None, 'fhr': None}

    def tmpfile(content, suffix, _data=data):
        fd, filename = tempfile.mkstemp(suffix=suffix)
        # It does not seem to be possible to use os.fdopen with 'rw' -- it
        # leads to "[Errno 9] Bad file descriptor"
        with os.fdopen(fd, 'w') as fhw:
            fhw.write(content)
        data['fhr'] = file(filename, 'r')
        data['filename'] = filename
        return data['fhr'], os.path.basename(filename)
    yield tmpfile
    data['fhr'].close()
    os.unlink(data['filename'])


# Fixtures to help asserting

@pytest.fixture(scope='function')
def sitemenu(browser):
    """Helper fixture to test the selections in the site menu.

    USABLE IN TESTS.

    """
    return icemac.addressbook.testing.SiteMenu


@pytest.fixture(scope='function')
def assert_address_book(address_book):
    """Fixture returning an object providing a custom address book asserts."""
    return icemac.addressbook.testing.AddressBookAssertions(address_book)


# Factory fixtures to create objects:

@pytest.fixture(scope='function')
def AddressBookFactory(addressBookConnectionF):
    """Create an address book in the root folder."""
    def create_addressbook(name, title=None):
        return icemac.addressbook.testing.create_addressbook(
            addressBookConnectionF.rootFolder, name, title)
    return create_addressbook

# generally usable helper fixtures:


@pytest.fixture(scope='session')
def DateTime():
    """Fixture to ease handling of datetime objects."""
    return DateTimeClass()


# Infrastructure fixtures

@pytest.yield_fixture(scope='session')
def zcmlS():
    """Load ZCML on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'AddressBook', __name__, icemac.addressbook, [])
    layer.setUp()
    yield layer
    layer.tearDown()
    # Needed so another ZCML layer can be run.
    zope.browserpage.metaconfigure.clear()
    zope.principalregistry.principalregistry.principalRegistry._clear()


@pytest.yield_fixture(scope='session')
def zodbS(zcmlS):
    """Create an empty test ZODB."""
    for zodb in icemac.addressbook.testing.pyTestEmptyZodbFixture():
        yield zodb


@pytest.yield_fixture(scope='session')
def addressBookS(zodbS):
    """Create an address book for the session."""
    for zodb in pyTestAddressBookFixture(zodbS, 'AddressBookS'):
        yield zodb


@pytest.yield_fixture(scope='function')
def addressBookConnectionF(addressBookS):
    """Get the connection to the right demo storage."""
    for connection in pyTestStackDemoStorage(addressBookS, 'AddressBookF'):
        yield connection


@pytest.yield_fixture(scope='function')
def translationZcmlF(zcmlS):
    """Load ZCML including locales packages on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'TranslatedAddressBook', __name__, icemac.addressbook, [zcmlS],
        filename='translationtesting.zcml')
    layer.setUp()
    yield layer
    layer.tearDown()


@pytest.yield_fixture(scope='session')
def personDataS(
        addressBookS, FullPersonFactory, PostalAddressFactory, KeywordFactory,
        PhoneNumberFactory, EMailAddressFactory, HomepageAddressFactory):
    """Create base data used in person tests."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            addressBookS, 'PersonDataSession'):
        for address_book in icemac.addressbook.conftest.site(connection):
            _create_person(
                address_book, FullPersonFactory, PostalAddressFactory,
                KeywordFactory, PhoneNumberFactory, EMailAddressFactory,
                HomepageAddressFactory)
            yield connection


# Helper functions and classes


def _create_person(
        address_book, FullPersonFactory, PostalAddressFactory, KeywordFactory,
        PhoneNumberFactory, EMailAddressFactory, HomepageAddressFactory, **kw):
    """Create a person, usable in tests."""
    KeywordFactory(address_book, u'friend')

    person = FullPersonFactory(
        address_book, u'Tester', first_name=u'Petra',
        keywords=[u'family', u'church'],
        postal__address_prefix=u'c/o Mama', postal__zip=u'88888',
        postal__street=u'Demoweg 23', postal__city=u'Testhausen',
        postal__country=gocept.country.db.Country('AT'),
        phone__number=u'+4901767654321', email__email=u'petra@example.com',
        homepage__url=b'http://petra.example.com', **kw)
    PostalAddressFactory(
        person, address_prefix=u'RST-Software',
        street=u'Forsterstraße 302a', zip=u'98344', city=u'Erfurt')
    PhoneNumberFactory(person, number=u'+4901761234567')
    EMailAddressFactory(person, email=u'pt@rst.example.edu')
    HomepageAddressFactory(person, url='http://www.rst.example.edu')
    transaction.commit()



from datetime import datetime
from mock import Mock, patch
from pytz import timezone
import icemac.addressbook.browser.table


def getSortKey(value):
    """Helper function to call the `getSortKey()` method of `DateTimeColumn.`.

    Assuming `value` is the raw value.
    """
    col = icemac.addressbook.browser.table.DateTimeColumn(
        Mock(), Mock(), Mock())
    getRawValue = (
        'icemac.addressbook.browser.table.DateTimeColumn.getRawValue')
    with patch(getRawValue) as getRawValue:
        getRawValue.return_value = value
        return col.getSortKey(Mock())


def test_table__DateTimeColumn__getSortKey__1():
    """`getSortKey` returns isoformat for naive datetime."""
    assert ('2012-02-02T21:57:00' ==
            getSortKey(datetime(2012, 2, 2, 21, 57)))


def test_table__DateTimeColumn__getSortKey__2():
    """`getSortKey` returns isoformat for timezone aware datetime."""
    assert ('2012-02-02T21:57:00+04:00' ==
            getSortKey(
                datetime(2012, 2, 2, 21, 57, tzinfo=timezone('Etc/GMT-4'))))


def test_table__DateTimeColumn__getSortKey__3():
    """`getSortKey` returns isoformat for `None` value to sort it down."""
    assert '9999-12-31T23:59:59' == getSortKey(None)



import zope.interface
import zope.publisher.interfaces.http
import zope.security.proxy


CHUNK_SIZE = 64 * 1024


class Download(object):

    def __call__(self):
        mime_type = self.context.mimeType
        if not mime_type:
            mime_type = 'application/octet-stream'
        self.request.response.setHeader('Content-Type', mime_type)

        self.request.response.setHeader('Content-Length', self.context.size)

        file_name = self.context.name.replace(' ', '_')
        self.request.response.setHeader(
            'Content-Disposition',
            'attachment; filename=%s' % file_name)

        return DownloadResult(self.context)


class DownloadResult(object):
    """Result object for a download request."""

    zope.interface.implements(zope.publisher.interfaces.http.IResult)

    def __init__(self, context):
        self._iter = bodyIterator(
            zope.security.proxy.removeSecurityProxy(context.openDetached()))

    def __iter__(self):
        return self._iter


def bodyIterator(f):
    while True:
        chunk = f.read(CHUNK_SIZE)
        if not chunk:
            f.close()
            raise StopIteration()
        yield chunk



from __future__ import unicode_literals
import pytest


@pytest.fixture(scope='function')
def pl_menu(address_book, browser, sitemenu):
    """Fixture to test the master data menu."""
    browser.login('editor')
    return sitemenu(
        browser, 0, 'Person list', browser.PERSONS_LIST_URL)


def test_menu__person_list_menu__1(pl_menu):
    """Asserting that the menu with the index 0 is `Person list`."""
    pl_menu.assert_correct_menu_item_is_tested()


def test_menu__person_list_menu__2(pl_menu):
    """The person list menu item is selected on the person list."""
    assert pl_menu.item_selected(pl_menu.menu_item_URL)


def test_menu__person_list_menu__3(address_book, pl_menu, PersonFactory):
    """The person list menu item is selected on the person export view."""
    PersonFactory(address_book, 'Tester')
    assert pl_menu.item_selected(pl_menu.browser.PERSON_EXPORT_URL)


def test_menu__person_list_menu__4(pl_menu):
    """The person list menu item is not selected on the search view."""
    assert not pl_menu.item_selected(pl_menu.browser.SEARCH_URL)


def test_menu__person_list_menu__5(pl_menu):
    """The person list menu item is selected on the person add form."""
    assert pl_menu.item_selected(pl_menu.browser.PERSON_ADD_URL)



from icemac.addressbook.address import HomePageAddress, PhoneNumber
from icemac.addressbook.address import PostalAddress, EMailAddress
from icemac.addressbook.address import default_attrib_name_to_entity
from icemac.addressbook.address import normalize_phone_number
from icemac.addressbook.interfaces import IEMailAddress, IPostalAddress, ITitle
from icemac.addressbook.interfaces import IHomePageAddress, IPhoneNumber
import pytest
import zope.interface.verify


def test_address__PostalAddress__1():
    """`PostalAddress` fulfills the `IPostalAddress` interface."""
    zope.interface.verify.verifyObject(IPostalAddress, PostalAddress())


def test_address__EMailAddress__1():
    """`EMailAddress` fulfills the `IEMailAddress` interface."""
    zope.interface.verify.verifyObject(IEMailAddress, EMailAddress())


def test_address__HomePageAddress__1():
    """`HomePageAddress` fulfills the `IHomePageAddress` interface."""
    zope.interface.verify.verifyObject(IHomePageAddress, HomePageAddress())


def test_address__PhoneNumber__1():
    """`PhoneNumber` fulfills the `IPhoneNumber` interface."""
    zope.interface.verify.verifyObject(IPhoneNumber, PhoneNumber())


def test_address__default_attrib_name_to_entity__1(stubEntities):
    """`default_attrib_name_to_entity` raises a ValueError for unknown name."""
    with pytest.raises(ValueError):
        default_attrib_name_to_entity('foo')


def test_address__default_attrib_name_to_entity__2(stubEntities):
    """`default_attrib_name_to_entity` returns the entity for a known name."""
    assert stubEntities.duck == default_attrib_name_to_entity('default_duck')


def test_address__normalize_phone_number__1():
    """`normalize_phone_number` returns a normalized number unchanged."""
    assert '+491234567890' == normalize_phone_number('+491234567890', '+49')


def test_address__normalize_phone_number__2():
    """`normalize_phone_number` keeps only numbers and a leading plus sign."""
    assert '+491234567890' == normalize_phone_number(
        '+49 (1234) 5678 - 90X', '+49')


def test_address__normalize_phone_number__3():
    """`normalize_phone_number` replaces a leading '0' by the country code."""
    assert '+491234567891' == normalize_phone_number('01234/5678-91', '+49')


def test_address__normalize_phone_number__4():
    """`normalize_phone_number` replaces only the first '0' by country code."""
    assert '+491234507090' == normalize_phone_number('01234/5070-90', '+49')


def test_address__normalize_phone_number__5():
    """`normalize_phone_number` does not replace '0' on empty country code."""
    assert '01234567891' == normalize_phone_number('01234/5678-91', '')


def test_address__normalize_phone_number__6():
    """`normalize_phone_number` replaces leading '00' by plus sign.

    The country code is ignored in this case.
    """
    assert '+421234567891' == normalize_phone_number(
        '0042-1234/5678-91', '+49')


def test_address__normalize_phone_number__7():
    """`normalize_phone_number` replaces only leading '00' by plus sign."""
    assert '+421234007891' == normalize_phone_number(
        '0042-1234/0078-91', '+49')


def test_address__email_address_of_person__1(
        address_book, PersonFactory, EMailAddressFactory):
    """Adaption to IEMailAddress returns the default address of the person."""
    person = PersonFactory(address_book, u'Tester')
    EMailAddressFactory(person, u'tester@exmaple.com', set_as_default=False)
    EMailAddressFactory(person, u't@exmaple.net', set_as_default=True)
    assert u't@exmaple.net' == IEMailAddress(person).email


def test_address__email_address_of_person__2(address_book, PersonFactory):
    """Missing default email address leads to a TypeError."""
    person = PersonFactory(address_book, u'Tester')
    with pytest.raises(TypeError) as err:
        IEMailAddress(person)
    assert 'Could not adapt' == err.value[0]


def test_address__postal_address_title__1(zcmlS):
    """A really empty postal address has a title of 'none'."""
    pa = PostalAddress()
    pa.country = None  # reset default value of `Germany`
    assert u'none' == ITitle(pa)


def test_address__postal_address_title__2(zcmlS):
    """`postal_address_title` displays set parts of the address."""
    pa = PostalAddress()
    pa.street = u'Papa street 3 a'
    assert u'Papa street 3 a, Germany' == ITitle(pa)


def test_address__postal_address_title__3(zcmlS):
    """`postal_address_title` is able to render the full address."""
    pa = PostalAddress()
    pa.address_prefix = u'c/o Mama'
    pa.street = u'Papa street 3 a'
    pa.zip = u'12345'
    pa.city = u'Dingshausen'
    assert (u'c/o Mama, Papa street 3 a, 12345, Dingshausen, Germany' ==
            ITitle(pa))


def test_address__email_address_title__1(zcmlS):
    """An empty email address has a title of 'none'."""
    assert u'none' == ITitle(EMailAddress())


def test_address__email_address_title__2(zcmlS):
    """`email_address_title` renders the email address if set."""
    ea = EMailAddress()
    ea.email = u'tester@example.org'
    assert u'tester@example.org' == ITitle(ea)


def test_address__home_page_address_title__1(zcmlS):
    """An empty home page address has a title of 'none'."""
    assert u'none' == ITitle(HomePageAddress())


def test_address__home_page_address_title__2(zcmlS):
    """`home_page_address_title` renders the home page URL if set."""
    hp = HomePageAddress()
    hp.url = 'http://www.example.org'
    assert u'http://www.example.org' == ITitle(hp)
    assert isinstance(ITitle(hp), unicode)


def test_address__phone_number_title__1(zcmlS):
    """An empty phone number has a title of 'none'."""
    assert u'none' == ITitle(PhoneNumber())


def test_address__phone_number_title__2(zcmlS):
    """`phone_number_title` renders the home page URL if set."""
    n = PhoneNumber()
    n.number = u'+017912345678'
    assert u'+017912345678' == ITitle(n)



import icemac.addressbook.interfaces
import icemac.addressbook.metadata.interfaces
import z3c.form.datamanager
import zope.annotation.interfaces
import zope.component
import zope.schema.interfaces
import zope.security.checker


class AnnotationField(z3c.form.datamanager.AttributeField):
    """Datamanager for data stored in annotations.

    Necessary to avoid forbidden attribute errors on annotations.  The
    code is a copy of z3c.form.datamanager.AttributeField but when the
    interface on the field is in ``self.no_security_proxy`` the
    security proxy gets removed.

    """

    zope.component.adapts(
        zope.annotation.interfaces.IAttributeAnnotatable,
        zope.schema.interfaces.IField)

    no_security_proxy = (
        # interfaces those values are stored in annotations
        icemac.addressbook.interfaces.IUserFieldStorage,
        icemac.addressbook.metadata.interfaces.IEditor)

    @property
    def adapted_context(self):
        context = self.context
        if self.field.interface in self.no_security_proxy:
            context = zope.security.proxy.getObject(context)
        context = self.field.interface(context)
        return context

    def canWrite(self):
        """See z3c.form.interfaces.IDataManager"""
        if self.field.interface in self.no_security_proxy:
            # When values are stored in annotations
            # self.adapted_context is not security proxied due to
            # permission problems with annotations, so we have to
            # check whether the interaction may access __annotations__
            # on the original context.
            context = self.context
            field_name = '__annotations__'
        else:
            context = self.adapted_context
            field_name = self.field.__name__

        if isinstance(context, zope.security.checker.Proxy):
            return zope.security.checker.canWrite(context, field_name)
        return True



# -*- coding: utf-8 -*-
import icemac.addressbook.browser.base
import icemac.addressbook.browser.interfaces
import z3c.preference.browser
import zope.interface


@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IAddressBookBackground)
class CategoryEditForm(icemac.addressbook.browser.base.BaseEditForm,
                       z3c.preference.browser.CategoryEditForm):
    """Preference CategoryEditForm which uses CSS of address book."""

    id = 'prefs-category-edit-form'
    next_url = 'site'
    next_view = 'person-list.html'
    __init__ = z3c.preference.browser.CategoryEditForm.__init__


# Not implementing IAddressBookBackground here as this view is used in
# icemac.ab.calendar, too.
class PrefGroupEditForm(icemac.addressbook.browser.base.BaseEditForm,
                        z3c.preference.browser.EditForm):
    """Preference group EditForm which uses CSS of address book."""

    __init__ = z3c.preference.browser.EditForm.__init__

    def redirect_to_next_url(self):
        # Stay on form but reload the values from database in case of
        # cancel:
        self.request.response.redirect(self.request.URL)



# -*- coding: utf-8 -*-
from icemac.addressbook.interfaces import IKeyword
from icemac.addressbook.testing import WebdriverPageObjectBase, Webdriver
import pytest


class POEntities(WebdriverPageObjectBase):
    """Webdriver page object for the entities page."""

    paths = [
        'ENTITY_PERSON_LIST_FIELDS_URL',
    ]

    _ENTITY_SELECTOR = "//tr[{}]/td[1]"

    def get_pos(self, pos):
        return self._selenium.getText(self._ENTITY_SELECTOR.format(pos))

    def move_to(self, pos, target):
        self._selenium.dragAndDropToObject(
            self._ENTITY_SELECTOR.format(pos),
            self._ENTITY_SELECTOR.format(target))

    def save(self):
        self._selenium.click("entity-fields-save")


Webdriver.attach(POEntities, 'entities')


@pytest.mark.webdriver
@pytest.mark.flaky(reruns=3)
def test_FieldOrder__1_webdriver(address_book, webdriver):
    """The fields of the entity can be sorted."""
    entities = webdriver.entities
    webdriver.login('mgr', entities.ENTITY_PERSON_LIST_FIELDS_URL)
    assert 'first name' == entities.get_pos(1)
    assert 'keywords' == entities.get_pos(4)
    entities.move_to(4, 1)
    # After saving the field is still on the first position:
    entities.save()
    assert 'Saved sortorder.' == webdriver.message
    assert 'keywords' == entities.get_pos(1)


def test_FieldOrder__2(address_book, browser):
    """Displaying the entity it shows the meta data of the entity field order.

    Because entities itself are not persistent.

    """
    browser.login('mgr')
    browser.open(
        browser.ENTITIES_EDIT_URL +
        '/icemac.addressbook.address.PostalAddress/@@save-sortorder.html?'
        'f:list=country&f:list=address_prefix&f:list=street&f:list=city&'
        'f:list=zip')
    assert ('<span id="form-widgets-creator" '
            'class="text-widget textline-field">Manager' in browser.contents)


def test_Entities__1(address_book, UserFactory, browser):
    """A local administrator is able to edit the entities."""
    UserFactory(address_book, u'Ad', u'Min', u'ad@min.de', u'12345678',
                ['Administrator'])
    # Log in as the local administrator and navigate to the edit entities
    # section of the master data:
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getControl('User Name').value = 'ad@min.de'
    browser.getControl('Password').value = '12345678'
    browser.getControl('Log in').click()
    assert 'You have been logged-in successfully.' == browser.message
    browser.getLink('Master data').click()
    browser.getLink('Entities').click()
    assert browser.ENTITIES_EDIT_URL == browser.url
    assert '<td>address book</td>' in browser.contents
    assert '<td>person</td>' in browser.contents


def test_Fields__1(address_book, browser):
    """Choice field values can contain non-ASCII characters."""
    # Create a user defined choice field and a values containing a non-ASCII
    # character:
    browser.login('mgr')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getLink('Master data').click()
    browser.getLink('Entities').click()
    browser.getLink('Edit fields', index=1).click()
    assert browser.url.endswith(
        '++attribute++entities/icemac.addressbook.person.Person')
    browser.getLink('field').click()
    browser.getControl('type').displayValue = ['choice']
    browser.getControl('title').value = 'Ümläuttößt'
    browser.getControl('Add', index=0).click()
    assert [] == browser.message
    browser.getControl('value').value = 'Küche'
    browser.getControl('Add', index=1).click()
    assert u'"\xdcml\xe4utt\xf6\xdft" added.' == browser.message
    assert browser.url.endswith(
        '++attribute++entities/icemac.addressbook.person.Person')

    # Previously an error occurred when adding a new object for the entity with
    # the user defined field (a person in our case):
    browser.getLink('Person list').click()
    browser.getLink('person').click()
    assert ['No value', 'K\xc3\xbcche'] == browser.getControl(
        'Ümläuttößt').displayOptions


def test_Fields__2(address_book, FieldFactory, KeywordFactory, browser):
    """Deleting a value of a user defined choice field does not break the UI.

    Scenario: A user defined field is a choice. One of the values of this
    choice was chosen by a user. The administrator deletes this value in
    the choice. The form where the no longer existing choice value would
    be displayed does not break.

    """
    field_name = FieldFactory(address_book, IKeyword, 'Choice', u'kind',
                              values=[u'one', u'two', u'three']).__name__
    KeywordFactory(address_book, u'foo', **{field_name: u'two'})
    browser.login('mgr')
    browser.open(browser.KEYWORDS_LIST_URL)
    # Editing is possible, the entered values are stored:
    browser.getLink('foo').click()
    assert 'foo' == browser.getControl('keyword').value
    assert ['two'] == browser.getControl('kind').displayValue
    # When the selected value of the choice gets deleted, ...
    browser.getLink('Master data').click()
    browser.getLink('Entities').click()
    browser.getLink('Edit fields', index=7).click()
    assert browser.url.endswith(
        '++attribute++entities/icemac.addressbook.keyword.Keyword')
    browser.getLink('Edit').click()
    assert browser.url.endswith(
        '++attribute++entities/icemac.addressbook.keyword.Keyword/Field-1')
    remove = browser.getControl(name='form.widgets.values.1.remove')
    remove.getControl(value='1').click()
    browser.getControl(name='form.widgets.values.buttons.remove').click()
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    # the value is no longer displayed for the keyword:
    browser.getLink('Master data').click()
    browser.getLink('Keywords').click()
    browser.getLink('foo').click()
    assert ['No value', 'one', 'three'] == browser.getControl(
        'kind').displayOptions
    assert ['No value'] == browser.getControl('kind').displayValue



# -*- coding: utf-8 -*-
from icemac.addressbook.interfaces import IPerson
import datetime
import icemac.addressbook.conftest
import pytest
import transaction
import zope.component.hooks


# Fixtures to set-up infrastructure which are usable in tests:

@pytest.yield_fixture(scope='function')
def search_data(searchDataS):
    """Provide predefined search data, see `searchDataS`."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            searchDataS.zodb, 'search_data'):
        for address_book in icemac.addressbook.conftest.site(connection):
            yield address_book


@pytest.yield_fixture(scope='function')
def person_with_field_data(personWithFieldDataS):
    """Provide predefined person data, see `personWithFieldDataS`."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            personWithFieldDataS.zodb, 'PersonWithFieldFunction'):
        yield connection


# Infrastructure fixtures

@pytest.yield_fixture(scope='session')
def searchDataS(
        addressBookS, KeywordFactory, PersonFactory, FullPersonFactory):
    """Create base data used in search tests."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            addressBookS, 'SearchSession'):
        for address_book in icemac.addressbook.conftest.site(connection):
            KeywordFactory(address_book, u'work')
            KeywordFactory(address_book, u'anyone else')
            # person objects are not returned as addressBookConnectionF
            # puts a DemoStorage around the storage.
            FullPersonFactory(address_book, u'Hohmuth', keywords=[u'friends'])
            FullPersonFactory(
                address_book, u'Koch', keywords=[u'family', u'church'],
                birth_date=datetime.date(1952, 1, 24), notes=u'father-in-law')
            FullPersonFactory(
                address_book, u'Velleuer', keywords=[u'family', u'church'])
            FullPersonFactory(
                address_book, u'Liebig', keywords=[u'church'],
                notes=u'family')
            PersonFactory(address_book, u'Tester', first_name=u'Liese',
                          birth_date=datetime.date(1976, 11, 15))
            transaction.commit()
        yield connection


@pytest.yield_fixture(scope='session')
def personWithFieldDataS(
        addressBookS, FullPersonFactory, PostalAddressFactory, KeywordFactory,
        PhoneNumberFactory, EMailAddressFactory, HomepageAddressFactory,
        FieldFactory):
    """Create base data used in person tests."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            addressBookS, 'SearchSession'):
        address_book = connection.rootFolder['ab']
        with zope.component.hooks.site(address_book):
            field_name = FieldFactory(
                address_book, IPerson, 'TextLine', u'foobar').__name__
            icemac.addressbook.conftest._create_person(
                address_book, FullPersonFactory, PostalAddressFactory,
                KeywordFactory, PhoneNumberFactory, EMailAddressFactory,
                HomepageAddressFactory, **{field_name: u'my value'})
            yield connection



# -*- coding: utf-8 -*-
import setuptools


def read(filename):
    """Read a file in the current directory."""
    with open(filename) as f:
        return f.read()


version = '4.1.1'


setuptools.setup(
    name='icemac.addressbook',
    version=version,
    description="Multi user address book application",
    long_description="\n\n".join([
        read('README.rst'),
        read('HACKING.rst'),
        read('CHANGES.rst')]
    ),
    keywords='python address book addressbook zope3 zope application web '
             'phone number e-mail email home page homepage wsgi',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    download_url='https://pypi.org/project/icemac.addressbook',
    url='https://bitbucket.org/icemac/icemac.addressbook',
    license='ZPL 2.1',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Paste',
        'Framework :: Zope3',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Natural Language :: German',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Communications :: Email :: Address Book',
        'Topic :: Communications :: Telephony',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Office/Business :: Groupware',
    ],
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['icemac'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'ZConfig',
        'ZODB',
        'fanstatic',
        'gocept.country',
        'gocept.pagelet',
        'gocept.reference',
        'grokcore.component >= 2.5.1.dev1',
        'icalendar',
        'icemac.ab.locales [compile] >= 2.15',
        'icemac.truncatetext',
        'js.jquery',
        'js.jquery_timepicker_addon',
        'js.jqueryui',
        'setuptools',
        'xlwt',
        'z3c.authviewlet',
        'z3c.flashmessage',
        'z3c.form >= 3.3.1.dev0',
        'z3c.formui',
        'z3c.layer.pagelet',
        'z3c.locales [compile] >= 0.3.0',
        'z3c.menu.ready2go',
        'z3c.pagelet',
        'z3c.pagelet[chameleon]',
        'z3c.preference >= 0.2',
        'z3c.ptcompat',
        'z3c.table',
        'z3c.wizard',
        'zc.catalog',
        'zc.sourcefactory',
        'zdaemon',
        'zodbupdate',
        'zope.schema >= 4.3'
        'zope.app.appsetup',
        'zope.app.debug',
        'zope.app.locales >= 3.6',
        'zope.app.wsgi',
        'zope.browserpage',
        'zope.copypastemove',
        'zope.generations',
        'zope.globalrequest',
        'zope.i18n [zcml,compile]',
        'zope.interface',
        'zope.login',
        'zope.mimetype',
        'zope.pluggableauth',
        'zope.preference',
        'zope.principalannotation',
        'zope.principalregistry',
        'zope.publisher',
        'zope.securitypolicy >= 4.1',
        'zope.session',
        'zope.traversing',
    ],
    extras_require=dict(
        test=[
            'WebTest >= 2.0.25.dev0',
            'beautifulsoup4',
            'gocept.selenium >= 2.5',
            'gocept.testing',
            'mock',
            'gocept.jslint',
            'plone.testing',
            'xlrd',
            'z3c.etestbrowser',
            'zc.buildout',
            'zope.testbrowser >= 5.2.dev0',
            'zope.testing >= 3.8',
        ],
    ),
    entry_points="""
      [console_scripts]
      debug = icemac.addressbook.startup:interactive_debug_prompt
      addressbook = icemac.addressbook.startup:zdaemon_controller
      debug_ajax = icemac.addressbook.startup:zdaemon_controller_debug_ajax
      debug_pdb = icemac.addressbook.startup:zdaemon_controller_debug_pdb
      [paste.app_factory]
      main = icemac.addressbook.startup:application_factory
      [fanstatic.libraries]
      css = icemac.addressbook.browser.resource:css_lib
      js = icemac.addressbook.browser.resource:js_lib
      """
)



import icemac.addressbook.fieldsource
import icemac.addressbook.interfaces
import zope.app.appsetup.bootstrap
import zope.preference.default
import zope.preference.interfaces
import zope.component


def add(address_book):
    """Add a default preferences provider to the address book."""
    # Add a default preferences utility on the address book site.
    default_prefs = zope.app.appsetup.bootstrap.ensureUtility(
        address_book,
        zope.preference.interfaces.IDefaultPreferenceProvider, '',
        zope.preference.default.DefaultPreferenceProvider)
    if default_prefs is None:
        default_prefs = zope.component.getUtility(
            zope.preference.interfaces.IDefaultPreferenceProvider)

    # Set the defaults for the person lists.
    personLists = default_prefs.getDefaultPreferenceGroup('ab.personLists')
    personLists.columns = []
    # The default columns are person last name and person first name.
    person_entity = icemac.addressbook.interfaces.IEntity(
        icemac.addressbook.interfaces.IPerson)
    personLists.columns.append(
        icemac.addressbook.fieldsource.tokenize(person_entity, 'last_name'))
    personLists.columns.append(
        icemac.addressbook.fieldsource.tokenize(person_entity, 'first_name'))
    personLists._p_changed = True
    # The default sort column is person last name.
    personLists.order_by = icemac.addressbook.fieldsource.tokenize(
        person_entity, 'last_name')
    # The default sort direction is ascending
    personLists.sort_direction = 'ascending'

    # Set the defaults for the person list tab:
    prefs = default_prefs.getDefaultPreferenceGroup('ab.personListTab')
    prefs.batch_size = 20

    # Set the default time zone:
    prefs = default_prefs.getDefaultPreferenceGroup('ab.timeZone')
    prefs.time_zone = 'UTC'



import z3c.menu.ready2go
import zope.interface
import zope.viewlet.interfaces


class ISearchMenu(z3c.menu.ready2go.ISiteMenu):
    """Search menu."""


class ISearchForm(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearchResult(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearch(zope.interface.Interface):
    """A search."""

    def search(**kw):
        """Search for given keyword arguments.

        Returns iterable of results.
        """


class ISearchMenuItemOn(zope.interface.Interface):
    """List of view names.

    For theses names the search menu item in the main navigation should be
    highlighted.
    """



from ..errormessage import render_error
from icemac.addressbook.person import person_entity
from zope.i18n import translate


def test_errormessage__render_error__1(zcmlS):
    """It is able to render an error which does not belong to a field."""
    message = render_error(
        person_entity, field_name='', exc=RuntimeError('General Error!'))
    assert ('Unexpected error occurred: RuntimeError: General Error!' ==
            translate(message))



from mock import patch


def test_startpage__Dispatch__1(address_book, browser):
    """It redirects by default to the welcome page."""
    browser.login('visitor')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert browser.ADDRESS_BOOK_WELCOME_URL == browser.url


def test_startpage__Dispatch__2(address_book, browser):
    """It redirects to the page selected on the address book."""
    browser.login('mgr')
    browser.open(browser.ADDRESS_BOOK_EDIT_URL)
    browser.getControl('start page after log-in').displayValue = 'Search'
    browser.select_favicon()
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert browser.SEARCH_URL == browser.url


def test_startpage__Dispatch__3(address_book, browser):
    """If accessing the selected page is not allowed for the user it redirects.

    It redirects to the welcome page.

    """
    browser.login('mgr')
    browser.open(browser.ADDRESS_BOOK_EDIT_URL)
    browser.getControl('start page after log-in').displayValue = 'Search'
    browser.select_favicon()
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message

    can_access_uri_part = (
        'icemac.addressbook.browser.addressbook.startpage.can_access_uri_part')
    with patch(can_access_uri_part, return_value=False):
        browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
        assert browser.ADDRESS_BOOK_WELCOME_URL == browser.url



# declare namespace package, we need this extended version here so the
# package can be installed on a vanilla python where setuptools is not
# installed. The global install.py calls functions inside this
# package.
try:
    import pkg_resources
    pkg_resources.declare_namespace(__name__)
except ImportError:  # pragma: no cover
    import pkgutil
    __path__ = pkgutil.extend_path(__path__, __name__)



import icemac.addressbook.browser.personlist
import icemac.addressbook.browser.search.result.base


class PersonTable(
        icemac.addressbook.browser.personlist.BasePersonList,
        icemac.addressbook.browser.search.result.base.BasePersonTable):
    """Result table displaying columns defined in user preferences."""

    values = (
        icemac.addressbook.browser.search.result.base.BasePersonTable.values)


class ExportForm(
        icemac.addressbook.browser.search.result.base.BaseSearchResultForm):
    """Export form showing a table of found results."""

    table_class = PersonTable



from icemac.addressbook.i18n import _
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.search.interfaces
import z3c.form.button
import z3c.form.field
import z3c.formui.form
import zope.interface


@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IAddressBookBackground)
class Search(object):
    """View to select a search."""

    show_headline = True
    form_explanation = u''


@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IAddressBookBackground)
class BaseView(zope.publisher.browser.BrowserView):
    """Base class for search views.

    Needs to extend from `zope.publisher.browser.BrowserView` as otherwise
    no search result handler can be found (the machinery expects this view
    to provide IBrowserView).

    """

    form_explanation = u''  # description of the search form displayed below it

    search_params = None
    show_headline = False

    @property
    def result(self):
        if not self.search_params:
            return
        return self.search()

    def search(self):
        search = icemac.addressbook.browser.search.interfaces.ISearch(self)
        return search.search(**self.search_params)


class BaseSearchForm(z3c.formui.form.Form):
    """Base class for the search form."""

    interface = NotImplemented  # to be set in child class

    ignoreContext = True
    formErrorsMessage = _('There were some errors.')

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface)

    @z3c.form.button.buttonAndHandler(_('Search'), name='search')
    def search(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.__parent__.search_params = data


class BaseSearch(object):
    """Base class for search adapter."""

    zope.interface.implements(
        icemac.addressbook.browser.search.interfaces.ISearch)

    def __init__(self, *args):
        pass

    def search(self, **kw):
        raise NotImplementedError()



from ..base import BaseSearch
from ..interfaces import ISearch
from zope.interface.verify import verifyObject
import pytest


def test_base__Search__1(address_book, browser):
    """Global navigation provides a link to the search."""
    browser.login('visitor')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getLink('Search').click()
    assert browser.SEARCH_URL == browser.url
    # This link leads to the searches overview providing links to the offered
    # search types:
    assert (['Keyword search', 'Name search'] ==
            browser.etree.xpath('//ul[@class="bullet"]/li/a/span/text()'))


def test_base__BaseSearch__1():
    """It implements `ISearch`."""
    assert verifyObject(ISearch, BaseSearch())


def test_base__BaseSearch__search__1():
    """It needs to be implemented by its subclasses."""
    base_search = BaseSearch()
    with pytest.raises(NotImplementedError):
        base_search.search()



import pytest


@pytest.mark.webdriver
def test_preferences__1_webdriver(address_book, webdriver):
    """Testing JS funtions of preferences frontend."""
    prefs = webdriver.prefs
    webdriver.login('mgr', prefs.PREFS_URL)
    # Preference groups are closed by default, fields are not visible:
    prefs.wait_for_fields_visible(False)
    prefs.toggle_group('personLists')
    prefs.wait_for_fields_visible(True)
    # Clicking on a form element does not close the group:
    prefs.select_column('person -- birth date')
    prefs.wait_for_fields_visible(True)
    prefs.toggle_group('personLists')
    prefs.wait_for_fields_visible(False)



import icemac.addressbook.interfaces
import icemac.addressbook.namechooser.interfaces
import zope.generations.utility
import zope.location.interfaces
import zope.proxy


def evolve(context):
    """Update persistent INameSuffix adapter for python 2.6:

    Provide ILocation interface, so zope.annotation.factory does not return a
    location proxy, where unicode function is not able to find __unicode__
    method any more.

    """
    root = zope.generations.utility.getRootFolder(context)
    addressbooks = zope.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IAddressBook)
    for addressbook in addressbooks:
        name_suffix = icemac.addressbook.namechooser.interfaces.INameSuffix(
            addressbook.entities)
        name_suffix = zope.proxy.getProxiedObject(name_suffix)
        zope.interface.directlyProvides(name_suffix,
                                        zope.location.interfaces.ILocation)
        name_suffix.__parent__ = addressbook.entities
        name_suffix.__name__ = 'icemac.namechooser.DontReuseNames.NameSuffix'



# -*- coding: utf-8 -*-
import collections
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import icemac.addressbook.orderstorage
import persistent
import plone.testing.zca
import pytest
import zope.component
import zope.component.globalregistry
import zope.interface


# ready to use fixtures


@pytest.fixture(scope='function',
                params=[icemac.addressbook.entities.Entities,
                        icemac.addressbook.entities.PersistentEntities],
                ids=lambda x: x.__name__)
def stubEntities(zopeComponentTestingF, request):
    """Provide some stub entities to test the entities classes."""
    entities_util = request.param()
    zope.component.provideUtility(
        entities_util, icemac.addressbook.interfaces.IEntities)
    cat = icemac.addressbook.entities.create_entity(
        u'Cat', ICat, Cat, default_attrib='default_cat')
    zope.component.provideUtility(cat, name=cat.class_name)
    duck = icemac.addressbook.entities.create_entity(
        u'Duck', IDuck, Duck, default_attrib='default_duck')
    zope.component.provideUtility(duck, name=duck.class_name)
    kwack = icemac.addressbook.entities.create_entity(
        u'Kwack', IKwack, Kwack, default_attrib='default_kwack')
    zope.component.provideUtility(kwack, name=kwack.class_name)
    return StubEntities(entities_util, cat, duck, kwack)


@pytest.fixture(scope='function')
def stubSortOrder(stubEntities):
    """Provide an order storage for the stub entities."""
    order_store = icemac.addressbook.orderstorage.OrderStorage()
    order_store.add(
        stubEntities.cat.name, icemac.addressbook.interfaces.ENTITIES)
    order_store.add(
        stubEntities.kwack.name, icemac.addressbook.interfaces.ENTITIES)
    order_store.add(
        stubEntities.duck.name, icemac.addressbook.interfaces.ENTITIES)
    zope.component.provideUtility(
        order_store, icemac.addressbook.interfaces.IOrderStorage)
    entity_order = icemac.addressbook.entities.EntityOrder()
    zope.component.provideUtility(entity_order)
    return entity_order


@pytest.fixture(scope='function')
def entityAdapters(zopeComponentTestingF):
    """Provide some adapters to IEntity."""
    zope.component.provideAdapter(
        icemac.addressbook.entities.entity_by_name)
    zope.component.provideAdapter(
        icemac.addressbook.entities.entity_by_interface)
    zope.component.provideAdapter(
        icemac.addressbook.entities.entity_by_obj)


# Helper classes


StubEntities = collections.namedtuple(
    'StubEntities', ['entities', 'cat', 'duck', 'kwack'])


# Helper fixtures


@pytest.yield_fixture(scope='function')
def zopeComponentTestingF():
    """Set up and tear down zope.component architecture.

    Using an empty registry.
    """
    assert plone.testing.zca._REGISTRIES
    new = zope.component.globalregistry.BaseGlobalComponents(
        name='zopeComponentTestingF',
        bases=(plone.testing.zca._REGISTRIES[0],))
    plone.testing.zca.pushGlobalRegistry(new)
    yield
    current = plone.testing.zca._REGISTRIES[-1]
    current.__bases__ = (plone.testing.zca._REGISTRIES[-2],)
    plone.testing.zca.popGlobalRegistry()


# Helper classes

class IDuck(zope.interface.Interface):
    """A duck."""


class Duck(object):
    """Duck."""

    zope.interface.implements(IDuck)


class ICat(zope.interface.Interface):
    """A cat."""


class Cat(object):
    """Cat."""

    zope.interface.implements(ICat)


class IKwack(zope.interface.Interface):
    """A Kwack."""


class Kwack(persistent.Persistent):
    """Kwack."""

    zope.interface.implements(IKwack)


class IDog(zope.interface.Interface):
    """A dog."""



from icemac.addressbook.interfaces import IPerson, IEntity
from icemac.addressbook.testing import assert_forbidden
from zope.testbrowser.browser import LinkNotFoundError
import pytest
import zope.component.hooks


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__1(address_book, browser, username):
    """A user with a non-admin role has no edit link at master data."""
    browser.login(username)
    browser.open(browser.MASTER_DATA_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink('Entities')


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__2(address_book, browser, username):
    """A user with a non-admin role cannot access the view to edit entities."""
    assert_forbidden(browser, username, browser.ENTITIES_EDIT_URL)


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__3(address_book, browser, username):
    """A non-admin cannot access the view to edit fields of an entity."""
    assert_forbidden(browser, username, browser.ENTITY_PERSON_LIST_FIELDS_URL)


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__4(address_book, browser, username):
    """A non-admin cannot add a new user defined field."""
    assert_forbidden(browser, username, browser.ENTITY_PERSON_ADD_FIELD_URL)


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__5(address_book, FieldFactory, browser, username):
    """A a non-admin cannot edit a user defined field."""
    FieldFactory(address_book, IPerson, 'TextLine', u'baz')
    assert_forbidden(browser, username, browser.ENTITIY_PERSON_EDIT_FIELD_URL)


@pytest.mark.parametrize('username', ('editor', 'visitor'))
def test_fields__security__6(address_book, FieldFactory, browser, username):
    """A a non-admin cannot delete a user defined field."""
    FieldFactory(address_book, IPerson, 'TextLine', u'baz')
    assert_forbidden(
        browser, username, browser.ENTITIY_PERSON_DELETE_FIELD_URL)


def test_fields__List__fields__1(address_book, browser):
    """It omits fields which are tagged with `omit-from-field-list`."""
    browser.login('mgr')
    browser.open(browser.ENTITY_PERSON_LIST_FIELDS_URL)
    # Initially the field is displayed:
    assert 'birth date' in browser.contents
    try:
        IPerson['birth_date'].setTaggedValue('omit-from-field-list', True)
        browser.open(browser.ENTITY_PERSON_LIST_FIELDS_URL)
        # It is not displayed after setting `omit-from-field-list`:
        assert 'birth date' not in browser.contents
    finally:
        IPerson['birth_date'].setTaggedValue('omit-from-field-list', False)


def test_fields__AddForm__1(address_book, browser):
    """New fields can be added to entities.

    The user has to select an entity first and then choose the `add field`
    link:

    """
    browser.login('mgr')
    browser.open(browser.MASTER_DATA_URL)
    browser.getLink('Entities').click()
    assert browser.ENTITIES_EDIT_URL == browser.url
    browser.getLink('Edit fields', index=1).click()
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url
    browser.getLink('field').click()
    assert browser.ENTITY_PERSON_ADD_FIELD_URL == browser.url
    # When all invariants are satisfied, the form gets saved:
    browser.getControl('type').displayValue = ['text line']
    browser.getControl('title').value = 'baz'
    browser.getControl('notes').value = 'the baz field'
    browser.getControl(name='form.buttons.add').click()
    assert '"baz" added.' == browser.message
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url
    assert browser.ENTITIY_PERSON_DELETE_FIELD_URL in browser.contents


def test_fields__AddForm__2(address_book, browser):
    """Selecting the `choice` type requires values."""
    browser.login('mgr')
    browser.open(browser.ENTITY_PERSON_ADD_FIELD_URL)
    browser.getControl('type').displayValue = ['choice']
    browser.getControl('title').value = 'foobar'
    browser.getControl(name='form.buttons.add').click()
    assert [] == browser.message
    assert browser.ENTITY_PERSON_ADD_FIELD_URL == browser.url
    assert (
        '<div class="error">type "choice" requires at least one field value.'
        in browser.contents)


def test_fields__AddForm__3(address_book, FieldFactory, browser):
    """The ids of previously deleted fields are not reused."""
    FieldFactory(address_book, IPerson, 'TextLine', u'baz')
    browser.login('mgr')
    browser.open(browser.ENTITIY_PERSON_DELETE_FIELD_URL)
    browser.getControl('Yes').click()
    assert '"baz" deleted.' == browser.message

    browser.open(browser.ENTITY_PERSON_ADD_FIELD_URL)
    browser.getControl('type').displayValue = ['text line']
    browser.getControl('title').value = 'baz'
    browser.getControl(name='form.buttons.add').click()
    assert '"baz" added.' == browser.message
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url
    assert browser.getLink('Edit').url.endswith(
        '/icemac.addressbook.person.Person/Field-2')
    assert browser.ENTITIY_PERSON_EDIT_FIELD_URL != browser.getLink('Edit').url


def test_fields__EditForm__1(address_book, FieldFactory, browser):
    """When editing a user defined field the values are displayed.

    Values can be changed.
    """
    FieldFactory(
        address_book, IPerson, 'TextLine', u'baz', notes=u'the baz field')
    browser.login('mgr')
    browser.open(browser.ENTITY_PERSON_LIST_FIELDS_URL)
    browser.getLink('Edit').click()
    assert browser.ENTITIY_PERSON_EDIT_FIELD_URL == browser.url
    assert ['text line'] == browser.getControl('type').displayValue
    assert 'baz' == browser.getControl('title').value
    assert 'the baz field' == browser.getControl('notes').value

    browser.getControl('title').value = 'foobar'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url
    browser.getLink('Edit').click()
    assert 'foobar' == browser.getControl('title').value


def test_fields__EditForm__2(address_book, FieldFactory, browser):
    """When canceling the edit of a field the previous values are kept."""
    FieldFactory(address_book, IPerson, 'TextLine', u'foo')
    browser.login('mgr')
    browser.open(browser.ENTITIY_PERSON_EDIT_FIELD_URL)
    browser.getControl('title').value = 'barrrrrrr'
    browser.getControl('Cancel').click()
    assert 'No changes were applied.' == browser.message
    browser.getLink('Edit').click()
    assert 'foo' == browser.getControl('title').value


def test_fields__DeleteForm__1(address_book, FieldFactory, browser):
    """An `administrator` can delete a user defined field."""
    FieldFactory(address_book, IPerson, 'TextLine', u'foobar')
    browser.login('mgr')
    browser.open(browser.ENTITY_PERSON_LIST_FIELDS_URL)
    assert 'foobar' in browser.contents
    browser.getLink('Delete').click()
    assert browser.ENTITIY_PERSON_DELETE_FIELD_URL == browser.url
    browser.getControl('Yes').click()
    assert '"foobar" deleted.' == browser.message
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url
    assert 1 == browser.contents.count('foobar')  # the one from the message


def test_fields__DeleteForm__2(address_book, FieldFactory, browser):
    """`Administrator` can choose to cancel deleting a user defined field."""
    FieldFactory(address_book, IPerson, 'TextLine', u'foobar')
    browser.login('mgr')
    browser.open(browser.ENTITIY_PERSON_DELETE_FIELD_URL)
    browser.getControl('No, cancel').click()
    assert 'Deletion canceled.' == browser.message
    assert browser.ENTITY_PERSON_LIST_FIELDS_URL == browser.url


def test_fields__SaveSortorder__1(address_book, FieldFactory, browser):
    """The sort order of the fields of the entities can be changed globally.

    To show that the sort order includes user defined fields.
    The places where fields are displayed in order are changed accordingly.

    """
    FieldFactory(address_book, IPerson, 'Bool', u'grown-up')
    # There is no field order saved yet:
    assert [] == IEntity(IPerson).getFieldOrder()
    browser.login('mgr')
    # The sort order is changed using JavaScript (tested using Selenium). When
    # the user hits the `Save sortorder` button a URL is generated and sent to
    # the server:
    browser.open(browser.ENTITY_PERSON_LIST_FIELDS_URL +
                 '/@@save-sortorder.html?f:list=last_name&f:list=Field-1')
    assert 'Saved sortorder.' == browser.message
    with zope.component.hooks.site(address_book):
        assert [u'last_name', u'Field-1'] == IEntity(IPerson).getFieldOrder()



from __future__ import unicode_literals
import pytest


@pytest.fixture(scope='function')
def prefs_menu(address_book, browser, sitemenu):
    """Fixture to test the preferences menu."""
    browser.login('visitor')
    return sitemenu(browser, 2, 'Preferences', browser.PREFS_URL)


def test_menu__preferences_menu__1(prefs_menu):
    """Asserting that the menu with the index 2 is `Preferences`."""
    prefs_menu.assert_correct_menu_item_is_tested()


def test_menu__preferences_menu__2(prefs_menu):
    """The preferences menu item is selected on the preferences page."""
    assert prefs_menu.item_selected(prefs_menu.menu_item_URL)


def test_menu__preferences_menu__3(prefs_menu):
    """The preferences menu item is not selected on the person list."""
    assert not prefs_menu.item_selected(prefs_menu.browser.PERSONS_LIST_URL)


def test_menu__preferences_menu__4(prefs_menu):
    """The preferences menu item is selected on a sub preferences view."""
    assert prefs_menu.item_selected(prefs_menu.browser.PREFS_TIMEZONE_URL)



from icemac.addressbook.file.file import File
import icemac.addressbook.file.interfaces
import transaction
import zope.interface.verify


def test_file__File__1():
    """`File` conforms to `IFile`."""
    assert zope.interface.verify.verifyObject(
        icemac.addressbook.file.interfaces.IFile, File())


def test_file__File__size__1():
    """An empty file has the length of zero."""
    assert 0 == File().size


def test_file__File__size__2():
    """`size` counts the length of the file.."""
    assert 10 == File('1234567\n90').size


def test_file__File__open__1():
    """`open()` allows to read and write the underlying file."""
    f = File()
    fd = f.open('w')
    fd.write('qwertz.123')
    fd.close()
    try:
        fd = f.open('r')
        assert 'qwertz.123' == fd.read()
    finally:
        fd.close()


def test_file__File__data__1():
    """`data` always returns '' to trick z3c.form."""
    f = File()
    f.data = 'data'
    assert '' == f.data


def test_file__File__openDetached__1(empty_zodb):
    """`openDetached` returns file data disconnected from db connection."""
    # need to assign to tree, so commit works
    empty_zodb.rootFolder['f'] = f = File('data\n\nfoobar')
    # commit as openDetached expects a committed blob
    transaction.commit()
    try:
        fd = f.openDetached()
        assert 'data\n\nfoobar' == fd.read()
    finally:
        fd.close()


def test_file__File__replace__1(empty_zodb, tmpdir):
    """`replace` allows to replace a file by another using its filename."""
    # need to assign to tree, so commit works
    empty_zodb.rootFolder['f2'] = f = File('1234')
    fdw = tmpdir.join('other.file')
    fdw.write('6789\n0123')
    filename = str(fdw)
    f.replace(filename)
    transaction.commit()
    try:
        fdr = f.openDetached()
        assert '6789\n0123' == fdr.read()
    finally:
        fdr.close()



# -*- coding: utf-8 -*-
from StringIO import StringIO
from icemac.addressbook.browser.file.file import cleanup_filename
from icemac.addressbook.file.interfaces import IFile
from zope.testbrowser.browser import HTTPError, LinkNotFoundError
import pytest


def test_file__cleanup_filename__1():
    """cleanup_filename() handles `None` file name."""
    assert '<no name>' == cleanup_filename(None)


def test_file__cleanup_filename__2():
    """cleanup_filename() handles simple file name."""
    assert 'sample.txt' == cleanup_filename('sample.txt')


def test_file__cleanup_filename__3():
    """cleanup_filename() handles Unix file name."""
    assert 'sample.txt' == cleanup_filename('/home/user/sample.txt')


def test_file__cleanup_filename__4():
    """cleanup_filename() handles Windows file name."""
    assert 'sample.txt' == cleanup_filename(r'c:\Users\me\sample.txt')


def test_file__cleanup_filename__5():
    """cleanup_filename() handles UNC file name."""
    assert 'sample.txt' == cleanup_filename(r'\\server\mine\sample.txt')


def test_file__Add__1(address_book, FullPersonFactory, browser, tmpfile):
    """`Add` allows to add a file to a person."""
    FullPersonFactory(address_book, u'Tester')
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('file').click()
    assert browser.FILE_ADD_URL == browser.url
    fh, filename = tmpfile('File contents', '.txt')
    browser.getControl('file').add_file(fh, 'text/plain', filename)
    browser.getControl('Add').click()
    assert '"%s" added.' % filename == browser.message
    assert browser.PERSON_EDIT_URL == browser.url
    # The new file is shown there including a widget displaying the
    # normalized name and mime type:
    assert browser.getControl('file name').value == filename
    assert 'text/plain' == browser.getControl('Mime Type').value
    # But there is no file displayed:
    assert browser.getControl('file', index=1).value is None
    # There is also a download link for the file:
    browser.getLink('Download file').click()
    assert not browser.isHtml
    assert 'text/plain' == browser.headers['content-type']
    assert browser.headers[
        'content-disposition'] == 'attachment; filename=' + filename
    assert '13' == browser.headers['content-length']
    assert 'File contents' == browser.contents


def test_file__Add__2(address_book, FullPersonFactory, browser):
    """`Add` requires a file to be uploaded."""
    FullPersonFactory(address_book, u'Tester')
    browser.login('editor')
    browser.open(browser.FILE_ADD_URL)
    browser.getControl('Add').click()
    assert [] == browser.message
    assert browser.FILE_ADD_URL == browser.url
    assert ['Required input is missing.'] == browser.etree.xpath(
        '/descendant-or-self::ul[@class="errors"]/li/div/text()')


def test_file__Add__3(address_book, FullPersonFactory, browser):
    """`Add` allows to cancel adding."""
    FullPersonFactory(address_book, u'Tester')
    browser.login('editor')
    browser.open(browser.FILE_ADD_URL)
    browser.getControl('Cancel').click()
    assert 'Addition canceled.' == browser.message
    assert browser.PERSON_EDIT_URL == browser.url
    with pytest.raises(LookupError) as err:
        browser.getControl('file')
    assert unicode(err.value).startswith("label 'file'")


def test_file__Add__4(address_book, FieldFactory, FullPersonFactory, browser):
    """`'Add` displays user defined fields."""
    FieldFactory(address_book, IFile, 'Text', u'mynotes')
    FullPersonFactory(address_book, u'Tester')
    browser.login('editor')
    browser.open(browser.FILE_ADD_URL)
    browser.getControl('file').add_file(
        StringIO('Dear Tester, CU.'), 'text/plain', 'letter.txt')
    browser.getControl('mynotes').value = 'first letter'
    browser.getControl('Add').click()
    assert '"letter.txt" added.' == browser.message
    assert browser.PERSON_EDIT_URL == browser.url
    # The entered values got saved:
    assert 'text/plain' == browser.getControl('Mime Type').value
    assert 'letter.txt' == browser.getControl('name', index=2).value
    assert 'first letter' == browser.getControl('mynotes').value


def test_file__Add__5(address_book, FullPersonFactory, browser):
    """`Add` cannot be accessed by a visitor."""
    FullPersonFactory(address_book, u'Tester')
    browser.login('visitor')
    browser.open(browser.PERSON_EDIT_URL)
    # There is no add link
    with pytest.raises(LinkNotFoundError) as err:
        browser.getLink('file').click()
    # Direct access to the URL is forbidden:
    with pytest.raises(HTTPError) as err:
        browser.open(browser.FILE_ADD_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)


def test_file__Add__6(address_book, FullPersonFactory, browser, tmpfile):
    """It can handle larger files.

    Small files are not written to disk but handled via a StringIO.

    """
    FullPersonFactory(address_book, u'Tester')
    browser.login('editor')
    browser.open(browser.FILE_ADD_URL)
    fh, filename = tmpfile('File contents ' * 1024, '.txt')
    browser.getControl('file').add_file(fh, 'text/plain', filename)
    browser.getControl('Add').click()
    assert '"%s" added.' % filename == browser.message
    # A larger file can be downloaded successfully.
    browser.getLink('Download file').click()
    assert '14336' == browser.headers['content-length']
    assert browser.contents.startswith('File contents File contents')


def test_file__Add__Edit__Download__1(
        address_book, FullPersonFactory, FileFactory, browser, tmpfile):
    """It allows to upload, change and download more than one file."""
    # Upload
    FileFactory(FullPersonFactory(address_book, u'Test'), u'my-file.txt',
                data='boring text')
    browser.login('editor')
    browser.open(browser.FILE_ADD_URL)
    browser.getControl('file').add_file(
        StringIO('2nd file'), 'text/plain', '2nd.txt')
    browser.getControl('Add').click()
    assert '"2nd.txt" added.' == browser.message
    assert browser.PERSON_EDIT_URL == browser.url
    assert browser.getControl('name', index=3).value == '2nd.txt'
    assert 'text/plain' == browser.getControl('Mime Type', index=1).value
    assert browser.getControl('file', index=1).value is None
    # Download
    browser.getLink('Download file', index=1).click()
    assert 'text/plain' == browser.headers['content-type']
    assert (browser.headers['content-disposition'] ==
            'attachment; filename=2nd.txt')
    assert '8' == browser.headers['content-length']
    assert '2nd file' == browser.contents
    # Change
    browser.open(browser.PERSON_EDIT_URL)
    fh, filename = tmpfile('3rd try', '.txt')
    browser.getControl('file', index=3).add_file(fh, 'text/enriched', filename)
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    assert browser.PERSONS_LIST_URL == browser.url
    # Assert changes
    browser.open(browser.PERSON_EDIT_URL)
    assert browser.getControl('file name', index=1).value == filename
    assert 'text/enriched' == browser.getControl('Mime Type', index=1).value
    # Download changed file
    browser.getLink('Download file', index=1).click()
    assert 'text/enriched' == browser.headers['content-type']
    assert (browser.headers['content-disposition'] ==
            'attachment; filename=' + filename)
    assert '3rd try' == browser.contents


def test_file__DeleteFileForm__1(
        address_book, FullPersonFactory, FileFactory, browser):
    """A File can be deleted using the ``Delete single entry`` button."""
    person = FullPersonFactory(address_book, u'Test')
    FileFactory(person, u'my-file.txt', data='boring text')
    FileFactory(person, u'data.dat', data='<XML :/>')
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getControl('Delete single entry').click()
    assert browser.PERSON_DELETE_ENTRY_URL == browser.url
    assert ['postal address -- Germany',
            'phone number -- none',
            'e-mail address -- none',
            'home page address -- none',
            'file -- my-file.txt',
            'file -- data.dat'] == browser.getControl('Entries').displayOptions
    browser.getControl('Entries').getControl(
        'file -- my-file.txt').selected = True
    browser.getControl('Delete entry').click()
    # Before the entry gets deleted, an `are you sure` formular is presented
    # where the user can choose to cancel.
    assert browser.FILE_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert '"my-file.txt" deleted.' == browser.message
    assert browser.PERSON_EDIT_URL == browser.url
    # The deleted file is no longer displayed (there is now only one file):
    with pytest.raises(LookupError) as err:
        browser.getControl('file name', index=1)
    assert str(err.value).startswith(
        "label 'file name'\nIndex 1 out of range, available choices are 0...0")


def test_file__DeleteFileForm__2(
        address_book, FullPersonFactory, FileFactory, browser):
    """`DeleteFileForm` cannot be accessed by a visitor."""
    FileFactory(FullPersonFactory(address_book, u'Test'), u'my-file.txt',
                data='boring text')
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open(browser.FILE_DELETE_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)



def test_download__Download__1(
        address_book, FullPersonFactory, FileFactory, browser):
    """`Download` uses name and mimetype of the file.

    Spaces in the file name are replaced by underscores:

    """
    FileFactory(FullPersonFactory(address_book, u'Test'), u'my nice file.txt',
                data='boring text', mimeType='text/example')
    browser.login('visitor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('Download file').click()
    assert browser.FILE_DOWNLOAD_URL == browser.url
    assert 'text/example' == browser.headers['content-type']
    assert 'attachment; filename=my_nice_file.txt' == browser.headers[
        'content-disposition']
    assert 'boring text' == browser.contents


def test_download__Download__2(
        address_book, FullPersonFactory, FileFactory, browser):
    """`Download` renders a missing mimetype as ``application/octet-stream``.

    Spaces in the file name are replaced by underscores:

    """
    FileFactory(FullPersonFactory(address_book, u'Test'), u'my nice file.txt',
                mimeType=None)
    browser.login('visitor')
    browser.open(browser.FILE_DOWNLOAD_URL)
    assert 'application/octet-stream' == browser.headers['content-type']


def test_download__Download__3(
        address_book, FullPersonFactory, FileFactory, browser):
    """A visitor can download a file-"""
    FileFactory(FullPersonFactory(address_book, u'Test'), u'v.txt',
                data='visiting', mimeType='text/plain')
    browser.login('visitor')
    browser.open(browser.FILE_DOWNLOAD_URL)
    assert 'text/plain' == browser.headers['content-type']
    assert (browser.headers['content-disposition'] ==
            'attachment; filename=v.txt')
    assert 'visiting' == browser.contents



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.base
import icemac.addressbook.file.file
import icemac.addressbook.file.interfaces
import os.path
import z3c.form.field
import z3c.form.widget
import zope.component
import zope.mimetype.interfaces
import zope.security.proxy


def cleanup_filename(filename):
    """Clean the filename from a possibly existing path."""
    if filename is None:
        return _('<no name>')
    return filename.split('\\')[-1].split('/')[-1]


def update_blob(widget, file):
    """Update the mime type of the file.

    widget ... file upload widget
    file ... IFile instance to set the mime type on

    """
    if not widget.value:
        # no file uploaded
        return

    file.name = cleanup_filename(widget.filename)

    # Zope stores the path to the file in the FileUpload in the `name`
    # attribute only if the file is stored on disk, but not when it uses a
    # StringIO for smaller files:
    widget_file = widget.value
    file_name = getattr(widget_file, 'name', None)

    if file_name and os.path.exists(file_name):
        # If the file exists, use it.
        file.replace(file_name)
    else:
        # If not (python stores small files in a StringIO instead of a
        # real file on hard disk) update the contents.
        widget_file.seek(0)
        file.data = widget_file.read()
        widget_file.close()

    # get a sample of the file contents for fingerprinting
    fd = zope.security.proxy.removeSecurityProxy(file.open())
    data = fd.read(100)
    fd.close()

    content_type = widget.headers.get('Content-Type')
    if content_type == 'application/octet-stream':
        content_type = None
    mime_type_getter = zope.component.getUtility(
        zope.mimetype.interfaces.IMimeTypeGetter)
    mime_type = mime_type_getter(data=data, content_type=content_type,
                                 name=cleanup_filename(widget.filename))
    if not mime_type:
        mime_type = 'application/octet-stream'
    file.mimeType = mime_type


class Add(icemac.addressbook.browser.base.BaseAddForm):
    """Add a file."""

    label = _(u'Add new file')
    class_ = icemac.addressbook.file.file.File
    interface = icemac.addressbook.file.interfaces.IFile
    next_url = 'parent'

    @property
    def fields(self):
        return z3c.form.field.Fields(
            *icemac.addressbook.interfaces.IEntity(
                self.interface).getFieldValues()).omit(
            'name', 'mimeType', 'size')

    def create(self, data):
        file = super(Add, self).create(data)
        update_blob(self.widgets['data'], file)
        return file


# in the Add form file upload is required ...
IFile_data_required = z3c.form.widget.StaticWidgetAttribute(
    True, context=None, request=None, view=Add,
    field=icemac.addressbook.file.interfaces.IFile['data'], widget=None)

# ... but not in all other forms
IFile_data_not_required = z3c.form.widget.StaticWidgetAttribute(
    False, context=icemac.addressbook.file.interfaces.IFile,
    request=None, view=None,
    field=icemac.addressbook.file.interfaces.IFile['data'], widget=None)


class DeleteFileForm(icemac.addressbook.browser.base.BaseDeleteForm):
    """Are you sure question for deleting a file."""

    next_url = 'parent'
    label = _(u'Do you really want to delete this file?')
    interface = icemac.addressbook.file.interfaces.IFile
    field_names = ('name', )



from ..form import zope_i18n_pattern_to_jquery_pattern, FieldDescriptionAsHint
from datetime import datetime
from icemac.addressbook.interfaces import IKeyword
from icemac.addressbook.testing import WebdriverPageObjectBase, Webdriver
from mock import patch, Mock
from pytz import utc, timezone
import gocept.testing.mock
import icemac.addressbook.browser.form
import pytest
import zope.i18nmessageid
import zope.publisher.browser


@pytest.fixture(scope='function')
def DatetimeDataConverter(mockTimeZone):
    """Set up a `DatetimeDataConverter` object for tests."""
    widget = Mock()
    widget.request = zope.publisher.browser.TestRequest()
    field = Mock()
    field.missing_value = None
    return icemac.addressbook.browser.form.DatetimeDataConverter(field, widget)


@pytest.yield_fixture(scope='module')
def mockTimeZone():
    """Mock the timezone set in preferences to 'Etc/GMT-4'."""
    time_zone = ('icemac.addressbook.browser.form.'
                 'DatetimeDataConverter.time_zone')
    with patch(time_zone, gocept.testing.mock.Property()) as time_zone:
        time_zone.return_value = timezone('Etc/GMT-4')
        yield


def test_form__DatetimeDataConverter__toWidgetValue__1(DatetimeDataConverter):
    """`toWidgetValue` renders datetime with timezone localized."""
    assert u'13/02/01 21:20' == DatetimeDataConverter.toWidgetValue(
        datetime(2013, 2, 1, 17, 20, tzinfo=utc))


def test_form__DatetimeDataConverter__toWidgetValue__2(DatetimeDataConverter):
    """`toWidgetValue` renders naive datetime unchanged."""
    assert u'13/02/01 17:20' == DatetimeDataConverter.toWidgetValue(
        datetime(2013, 2, 1, 17, 20))


def test_form__DatetimeDataConverter__toWidgetValue__3(DatetimeDataConverter):
    """`toWidgetValue` renders `missing_value` as empty string."""
    assert u'' == DatetimeDataConverter.toWidgetValue(None)


def test_form__DatetimeDataConverter__toFieldValue__1(DatetimeDataConverter):
    """`toFieldValue` adds tzinfo."""
    assert (datetime(2013, 2, 1, 21, 20, tzinfo=timezone('Etc/GMT-4')) ==
            DatetimeDataConverter.toFieldValue(u'13/02/01 21:20'))


def test_form__DatetimeDataConverter__toFieldValue__2(DatetimeDataConverter):
    """`toFieldValue` leaves an empty value alone."""
    assert None is DatetimeDataConverter.toFieldValue(u'')


def test_form__zope_i18n_pattern_to_jquery_pattern__1():
    """Function converts German datetime format correctly."""
    assert ('dd.mm.y HH:mm' ==
            zope_i18n_pattern_to_jquery_pattern('datetime', 'dd.MM.yy HH:mm'))


def test_form__zope_i18n_pattern_to_jquery_pattern__2():
    """Function converts American datetime format correctly."""
    assert ('m/d/y hh:mm TT' ==
            zope_i18n_pattern_to_jquery_pattern('datetime', 'M/d/yy h:mm a'))


def test_form__zope_i18n_pattern_to_jquery_pattern__3():
    """Function converts American time format correctly."""
    assert ('h:mm TT' == zope_i18n_pattern_to_jquery_pattern('time', 'h:mm a'))


@pytest.fixture('function')
def po_datetime_webdriver():
    """Webdriver page object for the datetime widget."""
    class PODatetime(WebdriverPageObjectBase):
        paths = [
            'KEYWORD_EDIT_URL',
            'KEYWORDS_LIST_URL'
        ]

        def datetime_now(self):
            s = self._selenium
            # Activate the datetime field which opens the JavaScript calendar
            s.click('id=form-widgets-Field-1')
            # Click the `now` button:
            s.click("//button[@type='button']")
            # And the `done` button:
            s.click("xpath=(//button[@type='button'])[2]")

        def save(self):
            self._selenium.click("id=form-buttons-apply")

    Webdriver.attach(PODatetime, 'dt')
    yield
    Webdriver.detach(PODatetime, 'dt')


@pytest.mark.webdriver
@pytest.mark.parametrize('datatype', ['Datetime', 'Time'])
def test_form__Widget__1_webdriver(address_book, FieldFactory, KeywordFactory, datatype, po_datetime_webdriver, webdriver):  # noqa
    """`DatetimeWidget` renders a JavaScript calendar."""
    FieldFactory(address_book, IKeyword, datatype, u'my-field')
    KeywordFactory(address_book, u'foobar')
    dt = webdriver.dt
    webdriver.login('editor', dt.KEYWORD_EDIT_URL)
    webdriver.windowMaximize()
    dt.datetime_now()
    dt.save()
    # Successful apply leads back to keyword overview:
    assert dt.KEYWORDS_LIST_URL == webdriver.path


def test_form__TimeWidget__2(
        address_book, FieldFactory, KeywordFactory, browser):
    """It renders a JavaScript calendar. (test for coverage)"""
    FieldFactory(address_book, IKeyword, 'Time', u'my-field')
    KeywordFactory(address_book, u'foobar')
    browser.login('editor')
    browser.open(browser.KEYWORD_EDIT_URL)
    assert '"timeFormat": "HH:mm",' in browser.contents


@pytest.fixture('function')
def po_date_webdriver():
    """Webdriver page object for the date widget."""
    class PODate(WebdriverPageObjectBase):
        paths = [
            'PERSON_ADD_URL',
            'PERSONS_LIST_URL',
        ]

        def select_first_of_month(self):
            self._selenium.click('css=.date-field')
            self._selenium.click("link=1")

        def save(self):
            # Fill in required fields:
            self._selenium.type(
                'id=IcemacAddressbookPersonPerson-widgets-'
                'IcemacAddressbookPersonPerson-last_name', 'Tester')
            self._selenium.click("id=form-buttons-add")

    Webdriver.attach(PODate, 'date')
    yield
    Webdriver.detach(PODate, 'date')


@pytest.mark.webdriver
def test_form__DateWidget__1_webdriver(address_book, po_date_webdriver, webdriver):  # noqa
    """It renders a JavaScript calendar."""
    date = webdriver.date
    webdriver.login('editor', date.PERSON_ADD_URL)
    webdriver.windowMaximize()
    date.select_first_of_month()
    date.save()
    # Successful apply leads back to keyword overview
    assert date.PERSONS_LIST_URL == webdriver.path


def test_form__DateWidget__2(address_book, browser):
    """It renders a JavaScript calendar. (test for coverage)"""
    browser.login('editor')
    browser.open(browser.PERSON_ADD_URL)
    assert '"dateFormat": "yy M d "' in browser.contents


def test_form__FieldDescriptionAsHint__get__1(zcmlS):
    """It returns the ComputedWidgetAttribute `hint` if it is registered."""
    obj = FieldDescriptionAsHint(
        field=zope.dublincore.interfaces.IDCTimes['modified'],
        context=None, request=None, form=None, widget=None)
    result = obj.get()
    assert ('The date and time that the object was last modified '
            'converted to the displayed time zone.' == result)
    assert isinstance(result, zope.i18nmessageid.Message)


def test_form__FieldDescriptionAsHint__get__2(address_book, FieldFactory):
    """It returns the description of a user defined field.

    It replaces newlines by spaces as the description is put into a TextLine
    field later on.
    """
    field = FieldFactory(
        address_book, icemac.addressbook.interfaces.IKeyword, u'Bool',
        u'usable?', notes=u'Is\nthis\rkeyword\r\nusable?')
    obj = FieldDescriptionAsHint(
        field=zope.schema.interfaces.IField(field),
        context=None, request=None, form=None, widget=None)
    result = obj.get()
    assert ('Is this keyword  usable?' == result)
    assert isinstance(result, unicode)



import zodbupdate.update


zaaa = "zope.app.authentication.authentication "
zaai = "zope.app.authentication.interfaces "
zaap = "zope.app.authentication.principalfolder "
zaas = "zope.app.authentication.session "
zaci = "zope.app.container.interfaces "
zacs = "zope.app.component.site "
zp = "zope.pluggableauth"


renames = {
    "zope.app.catalog.catalog Catalog": "zope.catalog.catalog Catalog",
    "zope.app.intid IntIds": "zope.intid IntIds",
    zaaa + "PluggableAuthentication": zp + " PluggableAuthentication",
    zaai + "IAuthenticatorPlugin": zp + ".interfaces IAuthenticatorPlugin",
    zaap + "PrincipalFolder": zp + ".plugins.principalfolder PrincipalFolder",
    zaas + "SessionCredentials": zp + ".plugins.session SessionCredentials",
    zaci + "IContained": "zope.container.interfaces IContained",
    zacs + "LocalSiteManager": "zope.site.site LocalSiteManager",
    zacs + "SiteManagementFolder": "zope.site.site SiteManagementFolder",
    zacs + "_LocalAdapterRegistry": "zope.site.site _LocalAdapterRegistry",
}


def evolve(context):
    """Update instances made from classes of from zope.app.* packages to ...

    ... their zope.* equivalents.

    """
    storage = context.connection._storage
    zodbupdate.update.Updater(storage, renames=renames, debug=True)()



from icemac.addressbook.i18n import _
import z3c.form.interfaces
import z3c.formui.interfaces
import z3c.layer.pagelet
import z3c.preference.interfaces
import zope.interface
import zope.viewlet.interfaces


class IAddressBookLayer(z3c.form.interfaces.IFormLayer,
                        z3c.layer.pagelet.IPageletBrowserLayer,
                        z3c.preference.interfaces.IPreferenceLayer):
    """Address book browser layer with form support."""


class IAddressBookBrowserSkin(z3c.formui.interfaces.IDivFormLayer,
                              IAddressBookLayer):
    """The address book browser skin using the div-based layout."""


class IPersonCount(zope.interface.Interface):
    """Number of persons for deletion."""

    count = zope.schema.Int(title=_(u'number of persons'), required=False)
    notes = zope.schema.TextLine(title=_(u'notes'), required=False)


class IErrorMessage(zope.interface.Interface):
    """Render error message human readable."""

    def __unicode__():
        """Return the translateable error text."""


class IDateWidget(z3c.form.interfaces.ITextWidget):
    """Special date widget to be able to use a JavaScript picker."""


class IDatetimeWidget(z3c.form.interfaces.ITextWidget):
    """Special date and time widget to be able to use a JavaScript picker."""


class ITimeWidget(z3c.form.interfaces.ITextWidget):
    """Special time widget to be able to use a JavaScript picker."""


class IImageSelectWidget(z3c.form.interfaces.ISelectWidget):
    """Select widget displays images as selectables."""


class IFanstaticViewletManager(zope.viewlet.interfaces.IViewletManager):
    """Manager those viewlets which `need()` fanstatic resources."""


class IAttributeTraversalHook(zope.interface.Interface):
    """Hook to be called when traversing an attribute.

    The traversed object and the request are taken to get the subscription
    adapters registered for this interface.

    """


class IIconProviderInfo(zope.interface.Interface):
    """Information about a someone whose icons are used in the address book."""

    name = zope.interface.Attribute('Name of the person or company')
    url = zope.interface.Attribute(
        'URL of the website the icons where taken from')


class IAddressBookBackground(zope.interface.Interface):
    """Marker for views which should display the address book background."""



# -*- coding: utf-8 -*-
import os
from icemac.addressbook.principals.sources import role_source
from icemac.addressbook.testing import TestHTTPPublicationRequestFactory
from icemac.addressbook.testing import getRootFolder
from icemac.addressbook.testing import interpolate_insted_of_translate
import gocept.httpserverlayer.wsgi
import gocept.selenium
import gocept.selenium.wd_selenese
import icemac.addressbook
import icemac.addressbook.file.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.principals.principals
import icemac.addressbook.startup
import icemac.addressbook.testing
import icemac.addressbook.utils
import mock
import pytest
import zope.app.wsgi.testlayer
import zope.browserpage.metaconfigure
import zope.component.hooks
import zope.event
import zope.i18n
import zope.principalregistry.principalregistry
import zope.processlifetime
import zope.testbrowser.wsgi

# Integrate these fixtures using:
# pytest_plugins = 'icemac.addressbook.fixtures'
# in your `conftest.py`


# Fixtures ready to use in tests:


@pytest.yield_fixture(scope='function')
def webdriver(webdriverS, httpServerS):  # pragma: no cover (webdriver)
    """Fixture to run tests using Webdriver."""
    assert icemac.addressbook.testing.CURRENT_CONNECTION is not None, \
        "The `webdriver` fixture needs a database fixture like `address_book`."
    timeout = int(os.environ.get('GOCEPT_SELENIUM_TIMEOUT', 10))
    webdriver = icemac.addressbook.testing.Webdriver(
        gocept.selenium.wd_selenese.Selenese(
            webdriverS, httpServerS, timeout))
    # Allow any language setting in the Webdriver browser by falling back to
    # interpolation instead of translation:
    translate = 'zope.i18n.translationdomain.TranslationDomain.translate'
    getPreferredLanguages = (
        'zope.publisher.browser.BrowserLanguages.getPreferredLanguages')
    with mock.patch(translate, new=interpolate_insted_of_translate), \
            mock.patch(getPreferredLanguages, return_value=['en-us', 'en']):
        yield webdriver


# Factory fixtures to create objects:


@pytest.fixture(scope='session')
def FieldFactory():
    """Create a user defined field in the address book.

    type ... see values of .sources.FieldTypeSource

    To create values for a Choice field use: values=<list of values>
    """
    def create(address_book, iface, type, title, **kw):
        with zope.component.hooks.site(address_book):
            field = icemac.addressbook.utils.create_obj(
                icemac.addressbook.entities.Field, type=type, title=title,
                **kw)
            entity = icemac.addressbook.interfaces.IEntity(iface)
            return entity.getRawField(entity.addField(field))
    return create


@pytest.fixture(scope='session')
def KeywordFactory():
    """Create a keyword in the address book."""
    def create_keyword(address_book, title, **kw):
        return icemac.addressbook.testing.create(
            address_book, address_book.keywords,
            icemac.addressbook.interfaces.IKeyword, title=title, **kw)
    return create_keyword


@pytest.fixture(scope='session')
def PersonFactory(KeywordFactory):
    """Create a person in the address book."""
    def create_person(address_book, last_name, keywords=[], **kw):
        keywords_set = set()
        keywords_util = zope.component.getUtility(
            icemac.addressbook.interfaces.IKeywords)
        for candidate in keywords:
            if icemac.addressbook.interfaces.IKeyword.providedBy(candidate):
                keyword = candidate
            else:
                keyword = keywords_util.get_keyword_by_title(candidate)
                if keyword is None:
                    keyword = KeywordFactory(address_book, candidate)
            keywords_set.add(keyword)
        return icemac.addressbook.testing.create(
            address_book, address_book, icemac.addressbook.interfaces.IPerson,
            last_name=last_name, keywords=keywords_set, **kw)
    return create_person


@pytest.fixture(scope='session')
def FullPersonFactory(
        PersonFactory, PostalAddressFactory, EMailAddressFactory,
        PhoneNumberFactory, HomepageAddressFactory, KeywordFactory):
    """Create a fully defined person in the address book usable in browser.

    Use <entity_name>__<attrib> to write to numbers and addresses.
    Possible entity names: 'email', 'phone', 'postal', 'homepage'

    The `keywords` parameter accepts an iterable of either keyword objects or
    keyword titles. Not existing keywords are created.

    """
    def create_full_person(address_book, last_name, **kw):
        by_entity = {}
        for key, value in kw.items():
            entity, _, attrib = key.partition('__')
            if not attrib:
                # default to 'person' for not prefixed keys
                attrib = entity
                entity = 'person'
            by_entity.setdefault(entity, {})[attrib] = value

        person = PersonFactory(
            address_book, last_name=last_name, **by_entity.get('person', {}))
        PostalAddressFactory(
            person, set_as_default=True, **by_entity.get('postal', {}))
        EMailAddressFactory(
            person, set_as_default=True, **by_entity.get('email', {}))
        PhoneNumberFactory(
            person, set_as_default=True, **by_entity.get('phone', {}))
        HomepageAddressFactory(
            person, set_as_default=True, **by_entity.get('homepage', {}))
        return person
    return create_full_person


@pytest.fixture(scope='session')
def PostalAddressFactory():
    """Create a postal address for a person in the address book."""
    def create_postal_address(
            person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IPostalAddress,
            set_as_default=set_as_default, **kw)
    return create_postal_address


@pytest.fixture(scope='session')
def EMailAddressFactory():
    """Create a e-mail address for a person in the address book."""
    def create_email_address(person, email=None, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IEMailAddress,
            email=email, set_as_default=set_as_default, **kw)
    return create_email_address


@pytest.fixture(scope='session')
def PhoneNumberFactory():
    """Create a phone number for a person in the address book."""
    def create_phone_number(person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IPhoneNumber,
            set_as_default=set_as_default, **kw)
    return create_phone_number


@pytest.fixture(scope='session')
def HomepageAddressFactory():
    """Create a home page address for a person in the address book."""
    def create_home_page_address(person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IHomePageAddress,
            set_as_default=set_as_default, **kw)
    return create_home_page_address


@pytest.fixture(scope='session')
def FileFactory():
    """Create a file for a person in the address book."""
    def create_file(person, filename, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.file.interfaces.IFile, name=filename, **kw)
    return create_file


@pytest.fixture(scope='session')
def UserFactory(FullPersonFactory):
    """Create a user in the address book.

    roles ... list of role titles (!)
    """
    def create_user(
            address_book, first_name, last_name, email, password, roles,
            keywords=[], **kw):
        person = FullPersonFactory(
            address_book, first_name=first_name, last_name=last_name,
            email__email=email, keywords=keywords)
        role_factory = role_source.factory
        role_values = role_factory.getValues()
        selected_roles = []
        for role_title in roles:
            for candidate in role_values:
                if role_factory.getTitle(candidate) == role_title:
                    selected_roles.append(candidate)
                    break
            else:
                raise LookupError(
                    'Role title {!r} unknown.'.format(role_title))
        # Cannot use icemac.addressbook.testing.create() here because
        # `Principal` is not an entity.
        name = icemac.addressbook.utils.create_and_add(
            address_book.principals,
            icemac.addressbook.principals.principals.Principal,
            person=person, password=password, roles=selected_roles)
        return address_book.principals[name]
    return create_user


# generally usable helper fixtures:


@pytest.yield_fixture('function')
def TimeZonePrefFactory():
    """Factory to set the time zone in the preferences.

    Usage example:  TimeZonePrefFactory('Europe/Berlin')
    """
    patchers = []

    def set_time_zone_pref(time_zone_name):
        patcher = mock.patch(
            'icemac.addressbook.preferences.utils.get_time_zone_name',
            return_value=time_zone_name)
        patcher.start()
        patchers.append(patcher)
    yield set_time_zone_pref
    while patchers:
        patchers.pop().stop()


# Infrastructure fixtures


@pytest.fixture(scope='session')
def wsgiAppS():
    """Create a WSGI application. for the current package."""
    zope_appl = icemac.addressbook.startup.zope_application_factory()
    zope_appl.requestFactory = TestHTTPPublicationRequestFactory()
    return icemac.addressbook.startup.application_factory(
        zope_application=zope_appl)


@pytest.fixture(scope='session')
def browserWsgiAppS(wsgiAppS):
    """WSGI application layered with some middlewares for testbrowser."""
    return zope.testbrowser.wsgi.AuthorizationMiddleware(
        zope.app.wsgi.testlayer.TransactionMiddleware(
            getRootFolder, wsgiAppS))


@pytest.yield_fixture(scope='session')
def httpServerS(wsgiAppS):  # pragma: no cover (webriver)
    """Create a HTTPServer for the WSGI app."""
    server = gocept.httpserverlayer.wsgi.Layer()
    server.wsgi_app = wsgiAppS
    server.setUp()
    server.url = 'http://{}'.format(server['http_address'])
    yield server['http_address']
    server.tearDown()


@pytest.yield_fixture(scope='session')
def webdriverS(httpServerS):  # pragma: no cover (webdriver)
    """Open a browser for webriver tests."""
    layer = gocept.selenium.WebdriverLayer(name='WebdriverLayer')
    layer['http_address'] = httpServerS
    layer.setUp()
    yield layer['seleniumrc']
    layer.tearDown()



from icemac.addressbook.testing import WebdriverPageObjectBase, Webdriver
import pytest


def test_resources__1(address_book, browser):
    """Image resources can be delivered to the browser."""
    browser.open('http://localhost/++resource++img/Symbol-Information.png')
    assert '200 Ok' == browser.headers['status']


class POPerson(WebdriverPageObjectBase):
    """Webdriver page object for the person page."""

    paths = [
        'PERSON_EDIT_URL',
    ]

    def get_hints(self):
        return self._selenium.getText("css=div.hint")


Webdriver.attach(POPerson, 'person')


@pytest.mark.webdriver
def test_resources__form_js__render_field_hint__1_webdriver(address_book, FullPersonFactory, webdriver):  # noqa
    """It renders the titles of the fields as separate hint."""
    FullPersonFactory(address_book, u'Tester')
    person_page = webdriver.person
    webdriver.login('editor', person_page.PERSON_EDIT_URL)
    assert "e. g. company name or c/o" == person_page.get_hints()



# -*- coding: utf-8 -*-
from icemac.addressbook.interfaces import IOrderStorage
from icemac.addressbook.orderstorage import OrderStorage
import pytest
import zope.interface.verify


# Fixtures


@pytest.fixture('function')
def storage():
    """`OrderStorage` filled with some items."""
    storage = OrderStorage()
    storage.add('foo1', 'bar')
    storage.add('foo2', 'bar')
    storage.add('foo3', 'bar')
    storage.add('foo4', 'bar')
    storage.add('baz', 'fuz')
    return storage


def test_orderstorage__OrderStorage__1():
    """`OrderStorage` fulfills the `IOrderStorage` interface."""
    zope.interface.verify.verifyObject(IOrderStorage, OrderStorage())


def test_orderstorage__Orderstorage__add__1():
    """It allows to add objects to namespaces."""
    storage = OrderStorage()
    assert [] == list(storage.namespaces())

    storage.add('foo', 'bar')
    assert ['bar'] == list(storage.namespaces())
    assert ['foo'] == storage.byNamespace('bar')

    storage.add('foo2', 'bar2')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert ['foo'] == storage.byNamespace('bar')
    assert ['foo2'] == storage.byNamespace('bar2')

    storage.add('foo2', 'bar')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert ['foo', 'foo2'] == storage.byNamespace('bar')
    assert ['foo2'] == storage.byNamespace('bar2')


def test_orderstorage__Orderstorage__add__2():
    """It does not raise an error when adding an object twice."""
    storage = OrderStorage()
    storage.add('foo', 'bar')
    storage.add('foo', 'bar')
    assert ['bar'] == list(storage.namespaces())
    assert ['foo'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__add__3():
    """Adding duplicates does not change the sort order."""
    storage = OrderStorage()
    storage.add('foo1', 'bar')
    storage.add('foo3', 'bar')
    storage.add('foo2', 'bar')
    storage.add('foo3', 'bar')
    assert ['foo1', 'foo3', 'foo2'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__add__4():
    """It allows to add an object to different namespaces."""
    storage = OrderStorage()
    storage.add('foo', 'bar')
    storage.add('foo', 'bar2')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert ['foo'] == storage.byNamespace('bar')
    assert ['foo'] == storage.byNamespace('bar2')


def test_orderstorage__Orderstorage__remove__1():
    """It allows to remove objects from the order storage."""
    storage = OrderStorage()
    storage.add('foo', 'bar')
    storage.add('foo2', 'bar2')
    storage.add('foo2', 'bar')

    storage.remove('foo', 'bar')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert ['foo2'] == storage.byNamespace('bar')
    assert ['foo2'] == storage.byNamespace('bar2')

    storage.remove('foo2', 'bar2')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert ['foo2'] == storage.byNamespace('bar')
    assert [] == storage.byNamespace('bar2')

    storage.remove('foo2', 'bar')
    assert ['bar', 'bar2'] == sorted(storage.namespaces())
    assert [] == storage.byNamespace('bar')
    assert [] == storage.byNamespace('bar2')


def test_orderstorage__Orderstorage__remove__2():
    """It raises `KeyError` on remove of an obj from not existing namespace."""
    storage = OrderStorage()
    storage.add('fuz', 'baz')
    with pytest.raises(KeyError):
        storage.remove('foo', 'bar')


def test_orderstorage__Orderstorage__remove__3():
    """It raises `ValueError` on remove of a not existing object."""
    storage = OrderStorage()
    storage.add('fuz', 'bar')
    with pytest.raises(ValueError):
        storage.remove('foo', 'bar')


def test_orderstorage__Orderstorage__get__1():
    """It raises a `KeyError` for a not existing namespace."""
    with pytest.raises(KeyError):
        OrderStorage().get('foo1', 'bar')


def test_orderstorage__Orderstorage__get__2():
    """It raises a `KeyError` for a not existing object."""
    storage = OrderStorage()
    storage.add('foo1', 'bar')
    with pytest.raises(KeyError):
        storage.get('foo2', 'bar')


def test_orderstorage__Orderstorage__get__3():
    """It raises a `KeyError` if the object was added to another namespace."""
    storage = OrderStorage()
    storage.add('foo1', 'baz')
    with pytest.raises(KeyError):
        storage.get('foo1', 'bar')


def test_orderstorage__Orderstorage__get__4(storage):
    """It returns the position of the item in the list."""
    storage.add('foo2', 'baz')
    assert 1 == storage.get('foo2', 'bar')
    assert 0 == storage.get('foo2', 'baz')


def test_orderstorage__Orderstorage__truncate__1():
    """It does not create a new namespace when it does not exist."""
    storage = OrderStorage()
    storage.truncate('baz')
    assert [] == sorted(storage.namespaces())


def test_orderstorage__Orderstorage__truncate__2():
    """It does only truncate the specified namespace."""
    storage = OrderStorage()
    storage.add('foo1', 'bar')
    storage.add('foo2', 'bar')
    storage.add('foo1', 'baz')
    storage.truncate('bar')
    assert [] == storage.byNamespace('bar')
    assert ['foo1'] == storage.byNamespace('baz')


def test_orderstorage__Orderstorage__up__1(storage):
    """It moves the given item one position up."""
    assert ['foo1', 'foo2', 'foo3', 'foo4'] == storage.byNamespace('bar')
    storage.up('foo3', 'bar')
    assert ['foo1', 'foo3', 'foo2', 'foo4'] == storage.byNamespace('bar')

    assert ['baz'] == storage.byNamespace('fuz')
    assert 1 == storage.get('foo3', 'bar')

    storage.up('foo3', 'bar')
    assert ['foo3', 'foo1', 'foo2', 'foo4'] == storage.byNamespace('bar')
    assert 0 == storage.get('foo3', 'bar')


def test_orderstorage__Orderstorage__up__2(storage):
    """Moving the first element up leads to a `ValueError`."""
    with pytest.raises(ValueError):
        storage.up('foo1', 'bar')


def test_orderstorage__Orderstorage__up__3(storage):
    """It moves the given item `delta` positions up."""
    storage.up('foo3', 'bar', 2)
    assert ['foo3', 'foo1', 'foo2', 'foo4'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__up__4(storage):
    """It raises a `ValueError` when moving an element up too much."""
    with pytest.raises(ValueError):
        storage.up('foo2', 'bar', 2)


def test_orderstorage__Orderstorage__down__1(storage):
    """It moves the given item up."""
    storage.down('foo3', 'bar')
    assert ['foo1', 'foo2', 'foo4', 'foo3'] == storage.byNamespace('bar')

    assert ['baz'] == storage.byNamespace('fuz')
    assert 0 == storage.get('foo1', 'bar')

    storage.down('foo1', 'bar')
    assert ['foo2', 'foo1', 'foo4', 'foo3'] == storage.byNamespace('bar')
    assert 1 == storage.get('foo1', 'bar')


def test_orderstorage__Orderstorage__down__2(storage):
    """Moving the last element down leads to a `ValueError`."""
    with pytest.raises(ValueError):
        storage.down('foo4', 'bar')


def test_orderstorage__Orderstorage__down__3(storage):
    """It can move the given item `delta` positions down."""
    storage.down('foo2', 'bar', 2)
    assert ['foo1', 'foo3', 'foo4', 'foo2'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__down__4(storage):
    """Moving am element down too much leads to a `ValueError`."""
    with pytest.raises(ValueError):
        storage.down('foo3', 'bar', 2)


def test_orderstorage__Orderstorage__down__5(storage):
    """If delta is 0 nothing is moved."""
    storage.down('foo2', 'bar', 0)
    assert ['foo1', 'foo2', 'foo3', 'foo4'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__down__6(storage):
    """If delta is negative the item is not moved."""
    storage.down('foo2', 'bar', -2)
    assert ['foo1', 'foo2', 'foo3', 'foo4'] == storage.byNamespace('bar')


def test_orderstorage__Orderstorage__isFirst__1(storage):
    """It raises a `KeyError` if the item is not in the list."""
    with pytest.raises(KeyError):
        storage.isFirst('foo', 'fuz')


def test_orderstorage__Orderstorage__isFirst__2(storage):
    """It returns `False` if the item is not the first one."""
    assert not storage.isFirst('foo2', 'bar')
    assert not storage.isFirst('foo3', 'bar')
    assert not storage.isFirst('foo4', 'bar')


def test_orderstorage__Orderstorage__isFirst__3(storage):
    """It returns `True` if the item is the first one."""
    assert storage.isFirst('foo1', 'bar')
    assert storage.isFirst('baz', 'fuz')


def test_orderstorage__Orderstorage__isLast__1(storage):
    """It raises a `KeyError` if the item is not in the list."""
    with pytest.raises(KeyError):
        storage.isLast('foo', 'fuz')


def test_orderstorage__Orderstorage__isLast__2(storage):
    """It returns `False` if the item is not the last one."""
    assert not storage.isLast('foo1', 'bar')
    assert not storage.isLast('foo2', 'bar')
    assert not storage.isLast('foo3', 'bar')


def test_orderstorage__Orderstorage__isLast__3(storage):
    """It returns `True` if the item is the first one."""
    assert storage.isLast('foo4', 'bar')
    assert storage.isLast('baz', 'fuz')



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.base
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import z3c.flashmessage.interfaces
import z3c.table.column
import zope.container.traversal
import zope.location
import zope.publisher.interfaces.http
import zope.security.proxy
import zope.traversing.browser.absoluteurl


class UpLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """Column displaying an `up` link."""

    header = _('move-up-table-header', default='move')
    linkName = 'up.html'
    linkContent = _('move-up-table-cell', default='up')

    def renderCell(self, item):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        if order.isFirst(item):
            return u''
        return super(UpLinkColumn, self).renderCell(item)


class DownLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """Column displaying an `down` link."""

    header = _('move-down-table-header', default='move')
    linkName = 'down.html'
    linkContent = _('move-down-table-cell', default='down')

    def renderCell(self, item):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        if order.isLast(item):
            return u''
        return super(DownLinkColumn, self).renderCell(item)


class EditFieldsLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """LinkColumn only displaying a link for IMayHaveUserFields items."""

    linkContent = _(u'Edit fields')
    header = u''

    def renderCell(self, item):
        if icemac.addressbook.interfaces.IEditableEntity.providedBy(item):
            return super(EditFieldsLinkColumn, self).renderCell(item)
        return ''


class List(icemac.addressbook.browser.table.Table):
    """List existing entities."""

    sortOn = None  # do not sort rows

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, z3c.table.column.I18nGetAttrColumn, 'title', weight=10,
                header=_(u'Entity'), attrName='title'),
            z3c.table.column.addColumn(
                self, UpLinkColumn, 'up', weight=20),
            z3c.table.column.addColumn(
                self, DownLinkColumn, 'down', weight=30),
            z3c.table.column.addColumn(
                self, EditFieldsLinkColumn, 'fields', weight=100)]

    @property
    def values(self):
        # Need to remove security otherwise local administrators get
        # an unauthorized exception.
        for x in self.context.getEntities():
            yield zope.security.proxy.getObject(x)


class EntityAbsoluteURL(zope.traversing.browser.absoluteurl.AbsoluteURL):
    """AbsoluteURL adapter for an entity."""

    zope.component.adapts(icemac.addressbook.interfaces.IEntity,
                          zope.publisher.interfaces.http.IHTTPRequest)

    def __str__(self):
        # parent is the entities utility and name is the class name
        parent = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        url = str(zope.component.getMultiAdapter(
            (parent, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL))
        url += '/' + self.context.class_name
        return url


class EntitiesTraverser(zope.container.traversal.ItemTraverser):
    """Make entities located and traverable."""

    zope.interface.implementsOnly(zope.publisher.interfaces.IPublishTraverse)
    zope.component.adapts(icemac.addressbook.interfaces.IEntities,
                          zope.publisher.interfaces.http.IHTTPRequest)

    def publishTraverse(self, request, name):
        entity = zope.component.queryUtility(
            icemac.addressbook.interfaces.IEntity, name=name)
        if entity is not None:
            return zope.location.LocationProxy(entity, self.context, name)
        return super(EntitiesTraverser, self).publishTraverse(request, name)


class MoveBase(icemac.addressbook.browser.base.BaseView):
    """Base class for movement views."""

    direction = None  # set on sub-class

    def __call__(self):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        getattr(order, self.direction)(self.context)
        self.request.response.redirect(self.url(self.context.__parent__))
        message = _(unicode(self.message),
                    mapping=dict(entity=self.context.title))
        zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource).send(message)


class MoveUp(MoveBase):
    """Move entity up in entity order."""

    direction = 'up'
    message = _('Moved ${entity} up.')


class MoveDown(MoveBase):
    """Move entity down in entity order."""

    direction = 'down'
    message = _('Moved ${entity} down.')


entity_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    icemac.addressbook.interfaces.IEntities,
    icemac.addressbook.interfaces.IEntity,
    icemac.addressbook.interfaces.IField)
